@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2024, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -426,9 +426,9 @@ int RotateFlowFiles(time_t t_start, char *time_extension, FlowSource_t *fs, int
         }
 
         // log stats
-        LogInfo("Ident: '%s' Flows: %llu, Packets: %llu, Bytes: %llu, Sequence Errors: %u, Bad Packets: %u, Blocks: %u", fs->Ident,
-                (unsigned long long)nffile->stat_record->numflows, (unsigned long long)nffile->stat_record->numpackets,
-                (unsigned long long)nffile->stat_record->numbytes, nffile->stat_record->sequence_failure, fs->bad_packets, ReportBlocks());
+        LogInfo("Ident: '%s' Flows: %" PRIu64 ", Packets: %" PRIu64 ", Bytes: %" PRIu64 ", Sequence Errors: %" PRIu64 ", Bad Packets: %u, Blocks: %u",
+                fs->Ident, nffile->stat_record->numflows, nffile->stat_record->numpackets, nffile->stat_record->numbytes,
+                nffile->stat_record->sequence_failure, fs->bad_packets, ReportBlocks());
 
         // reset stats
         fs->bad_packets = 0;
@@ -547,7 +547,7 @@ void FlushExporterStats(FlowSource_t *fs) {
     size = sizeof(exporter_stats_record_t) + (fs->exporter_count - 1) * sizeof(struct exporter_stat_s);
     exporter_stats = (exporter_stats_record_t *)malloc(size);
     if (!exporter_stats) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return;
     }
     exporter_stats->header.type = ExporterStatRecordType;
@@ -571,8 +571,8 @@ void FlushExporterStats(FlowSource_t *fs) {
         exporter_stats->stat[i].packets = e->packets;
         exporter_stats->stat[i].flows = e->flows;
 #ifdef DEVEL
-        printf("Stat: SysID: %u, version: %u, ID: %2u, Packets: %llu, Flows: %llu, Sequence Failures: %u\n", e->info.sysid, e->info.version,
-               e->info.id, e->packets, e->flows, e->sequence_failure);
+        printf("Stat: SysID: %u, version: %u, ID: %2u, Packets: %" PRIu64 ", Flows: %" PRIu64 ", Sequence Failures: %u\n", e->info.sysid,
+               e->info.version, e->info.id, e->packets, e->flows, e->sequence_failure);
 
 #endif
         // reset counters
