@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2023, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -32,6 +32,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <inttypes.h>
 #include <signal.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -65,7 +66,11 @@ static uint32_t timeout = 0;
 
 static void PrepareDirLists(channel_t *channel);
 
+#if defined __FreeBSD__
+static int compare(const FTSENT *const *f1, const FTSENT *const *f2);
+#else
 static int compare(const FTSENT **f1, const FTSENT **f2);
+#endif
 
 static void IntHandler(int signal) {
     switch (signal) {
@@ -250,7 +255,11 @@ uint64_t ParseTimeDef(char *s, uint64_t *value) {
 
 }  // End of ParseTimeDef
 
+#if defined __FreeBSD__
+static int compare(const FTSENT *const *f1, const FTSENT *const *f2) { return strcmp((*f1)->fts_name, (*f2)->fts_name); }  // End of compare
+#else
 static int compare(const FTSENT **f1, const FTSENT **f2) { return strcmp((*f1)->fts_name, (*f2)->fts_name); }  // End of compare
+#endif
 
 void RescanDir(char *dir, dirstat_t *dirstat) {
     FTS *fts;
@@ -266,7 +275,7 @@ void RescanDir(char *dir, dirstat_t *dirstat) {
 
     fts = fts_open(path, FTS_LOGICAL, compare);
     if (!fts) {
-        LogError("fts_open() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("fts_open() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return;
     }
     while ((ftsent = fts_read(fts)) != NULL) {
@@ -417,7 +426,7 @@ void ExpireDir(char *dir, dirstat_t *dirstat, uint64_t maxsize, uint64_t maxlife
                             num_expired++;
                             dir_files--;
                         } else {
-                            LogError("unlink() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+                            LogError("unlink() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
                         }
                         continue;  // next file if file was unlinked
                     } else {
@@ -435,7 +444,7 @@ void ExpireDir(char *dir, dirstat_t *dirstat, uint64_t maxsize, uint64_t maxlife
                             num_expired++;
                             dir_files--;
                         } else {
-                            LogError("unlink() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+                            LogError("unlink() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
                         }
                         lifetime_done = 0;
                     } else {
@@ -461,7 +470,7 @@ void ExpireDir(char *dir, dirstat_t *dirstat, uint64_t maxsize, uint64_t maxlife
                         // directory is empty and can be deleted
                         dbg_printf("Will remove directory %s\n", ftsent->fts_path);
                         if (rmdir(ftsent->fts_path) != 0) {
-                            LogError("rmdir() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+                            LogError("rmdir() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
                         }
                     }
                     break;
@@ -504,7 +513,7 @@ static void PrepareDirLists(channel_t *channel) {
 
         current_channel->fts = fts_open(path, FTS_LOGICAL | FTS_NOCHDIR, compare);
         if (!current_channel->fts) {
-            LogError("fts_open() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+            LogError("fts_open() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             continue;
         }
 
@@ -604,7 +613,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
 
         if (!size_done) {
             // expire size-wise if needed
-            dbg_printf("	Size expire %llu %llu\n", current_stat->filesize, sizelimit);
+            dbg_printf("	Size expire %" PRIu64 " %" PRIu64 "\n", current_stat->filesize, sizelimit);
             if (current_stat->filesize > sizelimit) {
                 // need to delete this file
                 if (unlink(expire_channel->ftsent->fts_path) == 0) {
@@ -620,7 +629,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
                     expire_channel->ftsent->fts_number--;
 
                 } else {
-                    LogError("unlink() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+                    LogError("unlink() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
                 }
                 file_removed = 1;
             } else {
@@ -648,7 +657,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
                     expire_channel->ftsent->fts_number--;
 
                 } else {
-                    LogError("unlink() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+                    LogError("unlink() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
                 }
                 file_removed = 1;
             } else {
@@ -694,7 +703,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
                                 // directory is empty and can be deleted
                                 dbg_printf("Will remove directory %s\n", expire_channel->ftsent->fts_path);
                                 if (rmdir(expire_channel->ftsent->fts_path) != 0) {
-                                    LogError("rmdir() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+                                    LogError("rmdir() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
                                 }
                             }
                             break;
@@ -712,7 +721,7 @@ void ExpireProfile(channel_t *channel, dirstat_t *current_stat, uint64_t maxsize
             if (expire_channel->dirstat->numfiles) {
                 // if channel is empty, no files must be reported, but rebuild is done anyway
                 LogError("Inconsistency detected in channel %s. Will rebuild automatically.\n", expire_channel->datadir);
-                LogError("No more files found, but %llu expected.\n", expire_channel->dirstat->numfiles);
+                LogError("No more files found, but %" PRIu64 " expected.\n", expire_channel->dirstat->numfiles);
             }
             expire_channel->dirstat->numfiles = 0;
             expire_channel->dirstat->status = FORCE_REBUILD;
