@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2022, Peter Haag
+ *  Copyright (c) 2009-2024, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -65,8 +65,6 @@
 #include "nfdump.h"
 #include "nffile.h"
 #include "privsep.h"
-
-#define DEVEL 1
 #include "util.h"
 
 typedef struct launcher_message_s {
@@ -160,7 +158,7 @@ static char *cmd_expand(char *launch_process, launcher_args_t *launcher_args) {
                     s = NULL;
             }
             if (s) {
-                q = realloc(q, strlen(q) + strlen(s));
+                q = (char *)realloc(q, strlen(q) + strlen(s));
                 if (!q) {
                     LogError("realloc() error in %s:%i: %s", __FILE__, __LINE__, strerror(errno));
                     return NULL;
@@ -288,7 +286,7 @@ static void do_expire(char *datadir) {
         LogError("Error %i: can't access book keeping records", ret);
     }
 
-    LogInfo("Limits: Filesize %s, Lifetime %s, Watermark: %llu%%\n", dirstat->max_size ? ScaleValue(dirstat->max_size) : "<none>",
+    LogInfo("Limits: Filesize %s, Lifetime %s, Watermark: %llu%%", dirstat->max_size ? ScaleValue(dirstat->max_size) : "<none>",
             dirstat->max_lifetime ? ScaleTime(dirstat->max_lifetime) : "<none>", (unsigned long long)dirstat->low_water);
 
     LogInfo("Current size: %s, Current lifetime: %s, Number of files: %llu", ScaleValue(dirstat->filesize), ScaleTime(dirstat->last - dirstat->first),
@@ -323,7 +321,7 @@ void processMessage(message_t *message, launcher_args_t *launcher_args) {
     len += lm->lenFilename + lm->lenFlowdir + lm->lenIsotime + lm->lenIdent + lm->lenAlign;
 
     if (message->length < len) {
-        LogError("Message size error: Expected: %zu, have: %u\n", len, message->length);
+        LogError("Message size error: Expected: %zu, have: %u", len, message->length);
         return;
     }
 
