@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2023, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -88,7 +88,7 @@ int Unicast_receive_socket(const char *bindhost, const char *listenport, int fam
 
     error = getaddrinfo(bindhost, listenport, &hints, &res);
     if (error) {
-        LogError("getaddrinfo error: [%s]", gai_strerror(error));
+        LogError("getaddrinfo() can not resolve %s:%s - %s", bindhost == NULL ? "any" : bindhost, listenport, gai_strerror(error));
         return -1;
     }
 
@@ -173,7 +173,7 @@ int Unicast_send_socket(const char *hostname, const char *sendport, int family,
 
     error = getaddrinfo(hostname, sendport, &hints, &res);
     if (error) {
-        LogError("getaddrinfo() error: %s", gai_strerror(error));
+        LogError("getaddrinfo() can not resolve %s:%s - %s", hostname, sendport, gai_strerror(error));
         return -1;
     }
 
@@ -247,7 +247,7 @@ int Multicast_receive_socket(const char *hostname, const char *listenport, int f
     error = getaddrinfo(hostname, listenport, &hints, &res);
 
     if (error) {
-        LogError("getaddrinfo error:: [%s]", gai_strerror(error));
+        LogError("getaddrinfo() can not resolve %s:%s - %s", hostname == NULL ? "localhost" : hostname, listenport, gai_strerror(error));
         return -1;
     }
 
@@ -466,7 +466,7 @@ int Raw_send_socket(int sockbuflen) {
 
     if (sockbuflen > 0) {
         if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (void *)&sockbuflen, sizeof sockbuflen) == -1) {
-            LogError("setsockopt(SO_SNDBUF,%ld): %s", sockbuflen, strerror(errno));
+            LogError("setsockopt(SO_SNDBUF,%d): %s", sockbuflen, strerror(errno));
             close(sock);
             return 0;
         }
