@@ -111,6 +111,7 @@ static uint64_t mpls_any_function(void *dataPtr, uint32_t length, data_t data, r
 static uint64_t pblock_function(void *dataPtr, uint32_t length, data_t data, recordHandle_t *handle);
 static uint64_t mmASLookup_function(void *dataPtr, uint32_t length, data_t data, recordHandle_t *handle);
 static uint64_t torLookup_function(void *dataPtr, uint32_t length, data_t data, recordHandle_t *handle);
+static uint64_t ttlEqual_function(void *dataPtr, uint32_t length, data_t data, recordHandle_t *handle);
 
 /* flow pre-processing functions */
 static void *ssl_preproc(uint32_t length, data_t data, recordHandle_t *handle);
@@ -137,6 +138,7 @@ static struct flow_procs_map_s {
                             [FUNC_PBLOCK] = {"pblock", pblock_function},
                             [FUNC_MMAS_LOOKUP] = {"AS Lookup", mmASLookup_function},
                             [FUNC_TOR_LOOKUP] = {"TOR Lookup", torLookup_function},
+                            [FUNC_TTL_EQUAL] = {"min/max TTL equal", ttlEqual_function},
                             {NULL, NULL}};
 
 static struct preprocess_s {
@@ -316,6 +318,13 @@ static uint64_t torLookup_function(void *dataPtr, uint32_t length, data_t data,
     return isTor;
 }  // End of torLookup_function
 
+static uint64_t ttlEqual_function(void *dataPtr, uint32_t length, data_t data, recordHandle_t *recordHandle) {
+    EXipInfo_t *ipInfo = (EXipInfo_t *)recordHandle->extensionList[EXipInfoID];
+    if (ipInfo == NULL) return 0;
+
+    return ipInfo->minTTL == ipInfo->maxTTL;
+}  // End of ttlEqual_function
+
 static void *ssl_preproc(uint32_t length, data_t data, recordHandle_t *handle) {
     const uint8_t *payload = (uint8_t *)(handle->extensionList[EXinPayloadID]);
     if (payload == NULL) return NULL;
@@ -448,7 +457,7 @@ uint32_t NewElement(uint32_t extID, uint32_t offset, uint32_t length, uint64_t v
             exit(255);
         }
     }
-    dbg_printf("New element: extID: %u, offset: %u, length: %u, value: %llu\n", extID, offset, length, value);
+    dbg_printf("New element: extID: %u, offset: %u, length: %u, value: %" PRIu64 "\n", extID, offset, length, value);
 
     FilterTree[n] = (filterElement_t){
         .extID = extID,
@@ -830,21 +839,28 @@ static int RunExtendedFilter(const FilterEngine_t *engine, recordHandle_t *handl
 }  // End of RunFilter
 
 char *ReadFilter(char *filename) {
-    struct stat stat_buff;
-    if (stat(filename, &stat_buff)) {
-        LogError("Can't stat filter file '%s': %s", filename, strerror(errno));
-        exit(EXIT_FAILURE);
-    }
-    char *filter = (char *)malloc(stat_buff.st_size + 1);
-    if (!filter) {
-        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
-        exit(EXIT_FAILURE);
-    }
+    if (!CheckPath(filename, S_IFREG)) return NULL;
+
     int ffd = open(filename, O_RDONLY);
     if (ffd < 0) {
         LogError("Can't open filter file '%s': %s", filename, strerror(errno));
-        exit(EXIT_FAILURE);
+        return NULL;
     }
+
+    struct stat stat_buff;
+    if (fstat(ffd, &stat_buff)) {
+        LogError("stat() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+        close(ffd);
+        return NULL;
+    }
+
+    char *filter = (char *)malloc(stat_buff.st_size + 1);
+    if (!filter) {
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+        close(ffd);
+        return NULL;
+    }
+
     ssize_t ret = read(ffd, (void *)filter, stat_buff.st_size);
     if (ret < 0) {
         LogError("Error reading filter file %s: %s", filename, strerror(errno));
