@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2024, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -192,7 +192,8 @@ static const char *subdir_def[] = {"",  // default index 0 - no subdir hierarchy
 #define AcceptedFormatChar "YymdjHMsUWwuF"
 
 static mode_t mode, dir_mode;
-static const char *subdir_format;
+static const char *subdir_format = NULL;
+static int subdir_index = 0;
 
 static struct entry_filter_s {
     char *first_entry;
@@ -235,7 +236,11 @@ static int CheckTimeWindow(char *filename, timeWindow_t *searchWindow);
 
 /* Functions */
 
+#if defined __FreeBSD__
+static int compare(const FTSENT *const *f1, const FTSENT *const *f2) { return strcmp((*f1)->fts_name, (*f2)->fts_name); }  // End of compare
+#else
 static int compare(const FTSENT **f1, const FTSENT **f2) { return strcmp((*f1)->fts_name, (*f2)->fts_name); }  // End of compare
+#endif
 
 static void CleanPath(char *entry) {
     char *p, *q;
@@ -1026,6 +1031,7 @@ int InitHierPath(int num) {
     }
 
     subdir_format = subdir_def[i];
+    subdir_index = i;
 
     /*
      * The default file mode is a=rwx (0777) with selected permissions
@@ -1111,6 +1117,8 @@ char *GetSubDir(struct tm *now) {
 
 }  // End of GetSubDir
 
+int GetSubDirIndex(void) { return subdir_index; }
+
 int SetupSubDir(char *dir, char *subdir, char *error, size_t errlen) {
     char *p, path[MAXPATHLEN];
     struct stat stat_buf;
@@ -1215,19 +1223,8 @@ static int CheckTimeWindow(char *filename, timeWindow_t *searchWindow) {
         return 0;
     }
 
-    // if relative time window, calculate absolute time
-    if (searchWindow->first && searchWindow->first <= 604800) {
-        searchWindow->last = stat_record.firstseen / 1000LL + searchWindow->first;
-        searchWindow->first = stat_record.firstseen / 1000LL;
-    }
-    if (searchWindow->last && searchWindow->last <= 604800) {
-        searchWindow->first = stat_record.lastseen / 1000LL - searchWindow->last;
-        searchWindow->last = 0;
-    }
-
-    if (searchWindow->last && searchWindow->last < (stat_record.firstseen / 1000LL)) return 0;
-
-    if (searchWindow->first && searchWindow->first > (stat_record.lastseen / 1000LL)) return 0;
+    if (searchWindow->msecLast && searchWindow->msecLast < stat_record.firstseen) return 0;
+    if (searchWindow->msecFirst && searchWindow->msecFirst > stat_record.lastseen) return 0;
 
     return 1;
 
