@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2004-2024, Peter Haag
+ *  Copyright (c) 2004-2025, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -302,7 +302,7 @@ static int Compress_Block_LZO(dataBlock_t *in_block, dataBlock_t *out_block, siz
     r = lzo1x_1_compress(in, in_len, out, &out_len, wrkmem);
 
     if (r != LZO_E_OK) {
-        LogError("Compress_Block_LZO() error compression failed in %s line %d: LZ4 : %d", __FILE__, __LINE__, r);
+        LogError("Compress_Block_LZO() error compression failed in %s line %d: LZ0 : %d", __FILE__, __LINE__, r);
         return -1;
     }
 
@@ -586,7 +586,7 @@ static int ReadAppendix(nffile_t *nffile) {
             processed += record_header->size;
             buff_ptr += record_header->size;
             if (processed > block_header->size) {
-                LogError("Error processing appendix records: processed %u > block size %u", processed, block_header->size);
+                LogError("Error processing appendix records: processed %zu > block size %u", processed, block_header->size);
                 FreeDataBlock(block_header);
                 return 0;
             }
@@ -596,7 +596,7 @@ static int ReadAppendix(nffile_t *nffile) {
 
     // seek back to currentPos
     off_t backPosition = lseek(nffile->fd, currentPos, SEEK_SET);
-    dbg_printf("Reset position to %lld -> %lld\n", currentPos, backPosition);
+    dbg_printf("Reset position to %lld -> %lld\n", (long long)currentPos, (long long)backPosition);
     if (backPosition < 0) {
         LogError("lseek() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return 0;
@@ -643,6 +643,8 @@ static int WriteAppendix(nffile_t *nffile) {
 
     recordHeader->type = TYPE_STAT;
     recordHeader->size = sizeof(recordHeader_t) + sizeof(stat_record_t);
+    // in case of an empty stat record
+    if (nffile->stat_record->firstseen == 0x7fffffffffffffffLL) nffile->stat_record->firstseen = 0;
     memcpy(data, nffile->stat_record, sizeof(stat_record_t));
 
     block_header->NumRecords++;
@@ -729,13 +731,8 @@ static nffile_t *OpenFileStatic(char *filename, nffile_t *nffile) {
         return NULL;
     } else {
         // regular file
-        if (stat(filename, &stat_buf)) {
-            LogError("stat() '%s': %s", filename, strerror(errno));
-            return NULL;
-        }
-
-        if (!S_ISREG(stat_buf.st_mode)) {
-            LogError("'%s' is not a file", filename);
+        if (access(filename, F_OK) < 0) {
+            LogError("access() '%s': %s", filename, strerror(errno));
             return NULL;
         }
 
@@ -744,6 +741,11 @@ static nffile_t *OpenFileStatic(char *filename, nffile_t *nffile) {
             LogError("Error open file: %s", strerror(errno));
             return NULL;
         }
+
+        if (fstat(fd, &stat_buf) < 0) {
+            LogError("fstat() '%s': %s", filename, strerror(errno));
+            return NULL;
+        }
     }
 
     // initialise and/or allocate new nffile handle
@@ -1040,40 +1042,31 @@ nffile_t *AppendFile(char *filename) {
 } /* End of AppendFile */
 
 int RenameAppend(char *oldName, char *newName) {
-    struct stat fstat;
+    if (access(newName, F_OK) == 0) {
+        // file exists already - concat them
+        nffile_t *nffile_w = AppendFile(newName);
+        if (!nffile_w) return -1;
 
-    int ret = stat(newName, &fstat);
-    if (ret == 0) {
-        // path exists
-        if (S_ISREG(fstat.st_mode)) {
-            // file exists already - concat them
-            nffile_t *nffile_w = AppendFile(newName);
-            if (!nffile_w) return -1;
+        nffile_t *nffile_r = OpenFile(oldName, NULL);
+        if (!nffile_r) return 0;
 
-            nffile_t *nffile_r = OpenFile(oldName, NULL);
-            if (!nffile_r) return 0;
-
-            // append data blocks
-            while (1) {
-                dataBlock_t *block_header = queue_pop(nffile_r->processQueue);
-                if (block_header == QUEUE_CLOSED)  // EOF
-                    break;
-                queue_push(nffile_w->processQueue, block_header);
-            }
-            CloseFile(nffile_r);
-
-            // sum stat_records
-            SumStatRecords(nffile_w->stat_record, nffile_r->stat_record);
-            DisposeFile(nffile_r);
-
-            CloseUpdateFile(nffile_w);
-            DisposeFile(nffile_w);
-
-            return unlink(oldName);
-        } else {
-            LogError("Path exists and is not a regular file: %s", newName);
-            return -1;
+        // append data blocks
+        while (1) {
+            dataBlock_t *block_header = queue_pop(nffile_r->processQueue);
+            if (block_header == QUEUE_CLOSED)  // EOF
+                break;
+            queue_push(nffile_w->processQueue, block_header);
         }
+        CloseFile(nffile_r);
+
+        // sum stat_records
+        SumStatRecords(nffile_w->stat_record, nffile_r->stat_record);
+        DisposeFile(nffile_r);
+
+        CloseUpdateFile(nffile_w);
+        DisposeFile(nffile_w);
+
+        return unlink(oldName);
     } else {
         // does not exist
         return rename(oldName, newName);
@@ -1257,7 +1250,7 @@ static dataBlock_t *nfread(nffile_t *nffile) {
     if (ret != sizeof(dataBlock_t)) {
         // this is most likely a corrupt file
         FreeDataBlock(buff);
-        LogError("Corrupt data file: Read %i bytes, requested %u", ret, sizeof(dataBlock_t));
+        LogError("Corrupt data file: Read %zd bytes, requested %lu", ret, sizeof(dataBlock_t));
         return NULL;
     }
 
@@ -1317,7 +1310,7 @@ static dataBlock_t *nfread(nffile_t *nffile) {
     } else if (ret == -1) {  // ERROR
         LogError("read() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
     } else {
-        LogError("read() error: Short read: Expected: %u, received: %u\n", buff->size, ret);
+        LogError("read() error: Short read: Expected: %u, received: %zd\n", buff->size, ret);
     }
 
     FreeDataBlock(buff);
@@ -1643,31 +1636,33 @@ void ModifyCompressFile(int compress) {
 }  // End of ModifyCompressFile
 
 int QueryFile(char *filename, int verbose) {
-    int fd;
-    uint32_t totalRecords, numBlocks, type1, type2, type3, type4;
-    struct stat stat_buf;
-    ssize_t ret;
-
     dbg_printf("Query mode verbose: %d\n", verbose);
     if (!Init_nffile(1, NULL)) return 0;
 
+    uint32_t totalRecords, numBlocks, type1, type2, type3, type4;
     type1 = type2 = type3 = type4 = 0;
     totalRecords = numBlocks = 0;
 
-    if (stat(filename, &stat_buf)) {
-        LogError("Can't stat '%s': %s", filename, strerror(errno));
+    if (access(filename, R_OK) < 0) {
+        LogError("Can't read '%s': %s", filename, strerror(errno));
         return 0;
     }
 
-    fd = open(filename, O_RDONLY);
+    int fd = open(filename, O_RDONLY);
     if (fd < 0) {
         LogError("Error open file: %s", strerror(errno));
         return 0;
     }
 
+    struct stat stat_buf;
+    if (fstat(fd, &stat_buf)) {
+        LogError("stat() error on '%s': %s", filename, strerror(errno));
+        return 0;
+    }
+
     // assume fileHeaderV2_t
     fileHeaderV2_t fileHeader;
-    ret = read(fd, (void *)&fileHeader, sizeof(fileHeaderV2_t));
+    ssize_t ret = read(fd, (void *)&fileHeader, sizeof(fileHeaderV2_t));
     if (ret < 1) {
         LogError("read() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         close(fd);
@@ -1804,7 +1799,7 @@ int QueryFile(char *filename, int verbose) {
             return 0;
         }
         if (ret < sizeof(dataBlock_t)) {
-            LogError("Short read: Expected %u bytes, read: %i", sizeof(dataBlock_t), ret);
+            LogError("Short read: Expected %lu bytes, read: %zd", sizeof(dataBlock_t), ret);
             close(fd);
             return 0;
         }
@@ -1864,7 +1859,7 @@ int QueryFile(char *filename, int verbose) {
             return 0;
         }
         if (ret != readBlock->size) {
-            LogError("Short read: Expected %u bytes, read: %i", readBlock->size, ret);
+            LogError("Short read: Expected %u bytes, read: %zd", readBlock->size, ret);
             close(fd);
             return 0;
         }
@@ -1997,7 +1992,7 @@ int QueryFile(char *filename, int verbose) {
         if (i + 1 == fileHeader.NumBlocks) {
             off_t fsize = lseek(fd, 0, SEEK_CUR);
             if (fileHeader.appendixBlocks && fsize != fileHeader.offAppendix) {
-                LogError("Invalid appendix offset - Expected: %u, found: %u", fileHeader.offAppendix, fsize);
+                LogError("Invalid appendix offset - Expected: %ld, found: %ld", fileHeader.offAppendix, fsize);
                 close(fd);
                 return 0;
             }
@@ -2010,7 +2005,7 @@ int QueryFile(char *filename, int verbose) {
 
     off_t fsize = lseek(fd, 0, SEEK_CUR);
     if (fsize < stat_buf.st_size) {
-        LogError("Extra data detected after regular blocks: %i bytes", stat_buf.st_size - fsize);
+        LogError("Extra data detected after regular blocks: %ld bytes", stat_buf.st_size - fsize);
     }
 
     printf("\nTotal\n");
@@ -2084,7 +2079,7 @@ static int QueryFileV1(int fd, fileHeaderV2_t *fileHeaderV2) {
         return 0;
     }
     if (ret != sizeof(stat_recordV1_t)) {
-        LogError("Error reading v1 stat record - short read. Expected: %u, get %u", sizeof(stat_recordV1_t), ret);
+        LogError("Error reading v1 stat record - short read. Expected: %lu, get %zd", sizeof(stat_recordV1_t), ret);
         return 0;
     }
 
