@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2022, Peter Haag
+ *  Copyright (c) 2024-2025, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -32,6 +32,7 @@
 
 #include <arpa/inet.h>
 #include <errno.h>
+#include <inttypes.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -128,14 +129,14 @@ uint16_t *SetupSequencer(sequencer_t *sequencer, sequence_t *sequenceTable, uint
     }
     if (!hasVarInLength && !hasVarOutLength) {
         dbg_printf("SetupSequencer() Fixed length fields, found %u elements in %u sequences\n", sequencer->numElements, sequencer->numSequences);
-        dbg_printf("SetupSequencer() Calculated input length: %lu, output length: %lu\n", sequencer->inLength, sequencer->outLength);
+        dbg_printf("SetupSequencer() Calculated input length: %zu, output length: %zu\n", sequencer->inLength, sequencer->outLength);
     }
 
     // dynamically create extension list
     dbg_printf("Extensionlist:\n");
     uint16_t *extensionList = calloc(sequencer->numElements, sizeof(uint16_t));
     if (!extensionList) {
-        LogError("SetupSequencer: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("SetupSequencer: malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return NULL;
     }
     int j = 0;
@@ -419,7 +420,7 @@ int SequencerRun(sequencer_t *sequencer, const void *inBuff, size_t inSize, void
 #endif
             if (stackID && stack) {
                 stack[stackID] = valBuff[0];
-                dbg_printf("Stack value %llu in slot %u\n", (long long unsigned)valBuff[0], stackID);
+                dbg_printf("Stack value %" PRIu64 " in slot %u\n", valBuff[0], stackID);
             }
 
             switch (outLength) {
@@ -462,7 +463,7 @@ int SequencerRun(sequencer_t *sequencer, const void *inBuff, size_t inSize, void
 
     nestLevel--;
 #ifdef DEVEL
-    printf("[%u] End sequencer ID: %u, inputLength: %lu, processed: %u, outputLength: %u header size: %u\n", nestLevel, sequencer->templateID, inSize,
+    printf("[%u] End sequencer ID: %u, inputLength: %zu, processed: %u, outputLength: %u header size: %u\n", nestLevel, sequencer->templateID, inSize,
            totalInLength, totalOutLength, recordHeaderV3->size);
 #endif
 
