@@ -652,7 +652,7 @@ typedef struct EXlayer2_s {
 #define SIZEetherType MemberSize(EXlayer2_t, etherType)
 #define OFFipVersion offsetof(EXlayer2_t, ipVersion)
 #define SIZEipVersion MemberSize(EXlayer2_t, ipVersion)
-    uint8_t fill;
+    uint8_t fill[5];
 } EXlayer2_t;
 #define EXlayer2Size (sizeof(EXlayer2_t) + sizeof(elementHeader_t))
 
@@ -685,15 +685,18 @@ typedef struct EXnokiaNatString_s {
 
 #define EXipInfoID 42
 typedef struct EXipInfo_s {
-    uint8_t ttl;
-#define OFFipTTL offsetof(EXipInfo_t, ttl)
-#define SIZEipTTL MemberSize(EXipInfo_t, ttl)
+    uint8_t fill;
 #define flagMF 0x20
 #define flagDF 0x40
     uint8_t fragmentFlags;
 #define OFFfragmentFlags offsetof(EXipInfo_t, fragmentFlags)
 #define SIZEfragmentFlags MemberSize(EXipInfo_t, fragmentFlags)
-    uint16_t align;  // unused for nfpcapd
+    uint8_t minTTL;  // unused for nfpcapd
+    uint8_t maxTTL;  // unused for nfpcapd
+#define OFFminTTL offsetof(EXipInfo_t, minTTL)
+#define SIZEminTTL MemberSize(EXipInfo_t, minTTL)
+#define OFFmaxTTL offsetof(EXipInfo_t, maxTTL)
+#define SIZEmaxTTL MemberSize(EXipInfo_t, maxTTL)
 } EXipInfo_t;
 #define EXipInfoSize (sizeof(EXipInfo_t) + sizeof(elementHeader_t))
 
@@ -749,10 +752,20 @@ typedef struct EXipInfo_s {
     h->numElements++;                                                              \
     h->size += s;
 
+#define PushExtensionID(h, x, v)                                                   \
+    {                                                                              \
+        elementHeader_t *elementHeader = (elementHeader_t *)((void *)h + h->size); \
+        elementHeader->type = x;                                                   \
+        elementHeader->length = extensionTable[x].size;                            \
+    }                                                                              \
+    void *v = (void *)((void *)h + h->size + sizeof(elementHeader_t));             \
+    memset(v, 0, extensionTable[x].size - sizeof(elementHeader_t));                \
+    h->size += extensionTable[x].size;                                             \
+    h->numElements++;
+
 #define ExtensionLength(ext) (((elementHeader_t *)((void *)ext - sizeof(elementHeader_t)))->length - sizeof(elementHeader_t))
 
-#define EXTENSION(s) \
-    { s##ID, s##Size, #s }
+#define EXTENSION(s) {s##ID, s##Size, #s}
 
 static const struct extensionTable_s {
     uint32_t id;    // id number
