@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2012-2024, Peter Haag
+ *  Copyright (c) 2012-2025, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -64,6 +64,8 @@ enum {
     STACK_ICMPCODE,
     STACK_DSTPORT,
     STACK_SAMPLER,
+    STACK_SECFIRST,
+    STACK_SECLAST,
     STACK_MSECFIRST,
     STACK_MSECLAST,
     STACK_DURATION,
@@ -155,6 +157,8 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_bgpSourceAsNumber, SIZEsrcAS, NumberCopy, EXasRoutingID, OFFsrcAS, STACK_NONE, "src AS"},
     {IPFIX_bgpDestinationAsNumber, SIZEdstAS, NumberCopy, EXasRoutingID, OFFdstAS, STACK_NONE, "dst AS"},
     {IPFIX_bgpNextHopIPv4Address, SIZEbgp4NextIP, NumberCopy, EXbgpNextHopV4ID, OFFbgp4NextIP, STACK_NONE, "IPv4 bgp next hop"},
+    {IPFIX_flowStartSeconds, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SECFIRST, "sec first seen"},
+    {IPFIX_flowEndSeconds, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SECLAST, "sec last seen"},
     {IPFIX_flowEndSysUpTime, Stack_ONLY, NumberCopy, EXnull, 0, STACK_MSECLAST, "msec last SysupTime"},
     {IPFIX_flowStartSysUpTime, Stack_ONLY, NumberCopy, EXnull, 0, STACK_MSECFIRST, "msec first SysupTime"},
     {IPFIX_SystemInitTimeMiliseconds, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SYSUPTIME, "SysupTime msec"},
@@ -172,6 +176,8 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_icmpCodeV4, SIZEicmpType, Stack_ONLY, EXgenericFlowID, OFFicmpCode, STACK_ICMPCODE, "icmp v4 code"},
     {IPFIX_icmpTypeV6, SIZEicmpCode, Stack_ONLY, EXgenericFlowID, OFFicmpType, STACK_ICMPTYPE, "icmp v6 type"},
     {IPFIX_icmpCodeV6, SIZEicmpType, Stack_ONLY, EXgenericFlowID, OFFicmpCode, STACK_ICMPCODE, "icmp v6 code"},
+    {IPFIX_MIN_TTL, SIZEminTTL, NumberCopy, EXipInfoID, OFFminTTL, STACK_NONE, "flow min TTL"},
+    {IPFIX_MAX_TTL, SIZEmaxTTL, NumberCopy, EXipInfoID, OFFmaxTTL, STACK_NONE, "flow max TTL"},
     {IPFIX_postIpClassOfService, SIZEdstTos, NumberCopy, EXflowMiscID, OFFdstTos, STACK_NONE, "post IP class of Service"},
     {IPFIX_SourceMacAddress, SIZEinSrcMac, NumberCopy, EXmacAddrID, OFFinSrcMac, STACK_NONE, "in src MAC addr"},
     {IPFIX_postDestinationMacAddress, SIZEoutDstMac, NumberCopy, EXmacAddrID, OFFoutDstMac, STACK_NONE, "out dst MAC addr"},
@@ -187,7 +193,7 @@ static const struct ipfixTranslationMap_s {
     {IPFIX_flowDirection, SIZEdir, NumberCopy, EXflowMiscID, OFFdir, STACK_NONE, "flow direction"},
     {IPFIX_biflowDirection, SIZEbiFlowDir, NumberCopy, EXflowMiscID, OFFbiFlowDir, STACK_NONE, "biFlow direction"},
     {IPFIX_flowEndReason, SIZEflowEndReason, NumberCopy, EXflowMiscID, OFFflowEndReason, STACK_NONE, "Flow end reason"},
-    {IPFIX_ipTTL, SIZEipTTL, NumberCopy, EXipInfoID, OFFipTTL, STACK_NONE, "IP ttl"},
+    {IPFIX_ipTTL, SIZEminTTL, NumberCopy, EXipInfoID, OFFminTTL, STACK_NONE, "flow min TTL"},
     {IPFIX_fragmentFlags, SIZEfragmentFlags, NumberCopy, EXipInfoID, OFFfragmentFlags, STACK_NONE, "IP fragment flags"},
     {IPFIX_ipNextHopIPv6Address, SIZENextHopV6IP, NumberCopy, EXipNextHopV6ID, OFFNextHopV6IP, STACK_NONE, "IPv6 next hop IP"},
     {IPFIX_bgpNextHopIPv6Address, SIZEbgp6NextIP, NumberCopy, EXbgpNextHopV6ID, OFFbgp6NextIP, STACK_NONE, "IPv6 bgp next hop IP"},
@@ -509,7 +515,7 @@ static exporterDomain_t *getExporter(FlowSource_t *fs, uint32_t ObservationDomai
 static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_record_t *sampler_record) {
     sampler_t *sampler;
 
-    dbg_printf("[%u] Insert Sampler: Exporter is 0x%llu\n", exporter->info.id, (long long unsigned)exporter);
+    dbg_printf("[%u] Insert Sampler: Exporter is 0x%p\n", exporter->info.id, (void *)exporter);
     if (!exporter->sampler) {
         // no samplers so far
         sampler = (sampler_t *)malloc(sizeof(sampler_t));
@@ -526,10 +532,10 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
         exporter->sampler = sampler;
 
         fs->dataBlock = AppendToBuffer(fs->nffile, fs->dataBlock, &(sampler->record), sampler->record.size);
-        LogInfo("Add new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id, sampler_record->algorithm,
-                sampler_record->packetInterval, sampler_record->spaceInterval);
-        dbg_printf("Add new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->id, sampler_record->algorithm,
-                   sampler_record->packetInterval, sampler_record->spaceInterval);
+        LogInfo("Add new sampler id: %" PRId64 ", algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id,
+                sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
+        dbg_printf("Add new sampler id: %" PRId64 ", algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->id,
+                   sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
 
     } else {
         sampler = exporter->sampler;
@@ -543,9 +549,9 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
                     sampler->record.algorithm = sampler_record->algorithm;
                     sampler->record.packetInterval = sampler_record->packetInterval;
                     sampler->record.spaceInterval = sampler_record->spaceInterval;
-                    LogInfo("Update existing sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id,
+                    LogInfo("Update existing sampler id: %" PRId64 ", algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id,
                             sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
-                    dbg_printf("Update existing sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->id,
+                    dbg_printf("Update existing sampler id: %" PRId64 " , algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->id,
                                sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
                 } else {
                     dbg_printf("Sampler unchanged!\n");
@@ -570,9 +576,9 @@ static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_
                 sampler->next = NULL;
 
                 fs->dataBlock = AppendToBuffer(fs->nffile, fs->dataBlock, &(sampler->record), sampler->record.size);
-                LogInfo("Append new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id,
+                LogInfo("Append new sampler id: %" PRId64 ", algorithm: %u, packet interval: %u, packet space: %u", sampler_record->id,
                         sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
-                dbg_printf("Append new sampler id: %lli, algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->id,
+                dbg_printf("Append new sampler id: %" PRId64 ", algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->id,
                            sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
                 break;
             }
@@ -1169,7 +1175,7 @@ static void Process_ipfix_option_templates(exporterDomain_t *exporter, void *opt
     }
     optionTemplate->optionSize = offset;
 
-    dbg_printf("\n[%u] Option size: %llu, flags: %llx\n", exporter->info.id, optionTemplate->optionSize, optionTemplate->flags);
+    dbg_printf("\n[%u] Option size: %" PRIu64 ", flags: %" PRIx64 "\n", exporter->info.id, optionTemplate->optionSize, optionTemplate->flags);
     if (optionTemplate->flags) {
         // if it exitsts - remove old template on exporter with same ID
         templateList_t *template = newTemplate(exporter, tableID);
@@ -1289,8 +1295,8 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
         // map file record to output buffer
         outBuff = GetCurrentCursor(fs->dataBlock);
 
-        dbg_printf("[%u] Process data record: %u addr: %llu, size_left: %u buff_avail: %u\n", exporter->info.id, processed_records,
-                   (long long unsigned)((ptrdiff_t)inBuff - (ptrdiff_t)data_flowset), size_left, buffAvail);
+        dbg_printf("[%u] Process data record: %u addr: %p, size_left: %u buff_avail: %u\n", exporter->info.id, processed_records,
+                   (void *)((ptrdiff_t)inBuff - (ptrdiff_t)data_flowset), size_left, buffAvail);
 
         // process record
         AddV3Header(outBuff, recordHeaderV3);
@@ -1335,10 +1341,8 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
                 break;
         }
 
-        dbg_printf(
-            "New record added with %u elements and size: %u, sequencer inLength: %lu, outLength: "
-            "%lu\n",
-            recordHeaderV3->numElements, recordHeaderV3->size, sequencer->inLength, sequencer->outLength);
+        dbg_printf("New record added with %u elements and size: %u, sequencer inLength: %zu, outLength: %zu\n", recordHeaderV3->numElements,
+                   recordHeaderV3->size, sequencer->inLength, sequencer->outLength);
 
         recordHeaderV3->engineType = stack[STACK_ENGINETYPE];
         recordHeaderV3->engineID = stack[STACK_ENGINEID];
@@ -1400,27 +1404,29 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
             packetInterval = overwriteSampler->record.packetInterval;
             spaceInterval = overwriteSampler->record.spaceInterval;
             SetFlag(recordHeaderV3->flags, V3_FLAG_SAMPLED);
-            dbg_printf("[%u] Overwrite sampling - packet interval: %llu, packet space: %llu\n", exporter->info.id, packetInterval, spaceInterval);
+            dbg_printf("[%u] Overwrite sampling - packet interval: %" PRIu64 ", packet space: %" PRIu64 "\n", exporter->info.id, packetInterval,
+                       spaceInterval);
         } else if (sampler) {
             // individual assigned sampler ID
             packetInterval = sampler->record.packetInterval;
             spaceInterval = sampler->record.spaceInterval;
             SetFlag(recordHeaderV3->flags, V3_FLAG_SAMPLED);
-            dbg_printf("[%u] Found assigned sampler ID %u - packet interval: %llu, packet space: %llu\n", exporter->info.id, sampler_id,
+            dbg_printf("[%u] Found assigned sampler ID %u - packet interval: %" PRIu64 ", packet space: %" PRIu64 "\n", exporter->info.id, sampler_id,
                        packetInterval, spaceInterval);
         } else if (genericSampler) {
             // global sampler ID
             packetInterval = genericSampler->record.packetInterval;
             spaceInterval = genericSampler->record.spaceInterval;
             SetFlag(recordHeaderV3->flags, V3_FLAG_SAMPLED);
-            dbg_printf("[%u] Found generic sampler - packet interval: %llu, packet space: %llu\n", exporter->info.id, packetInterval, spaceInterval);
+            dbg_printf("[%u] Found generic sampler - packet interval: %" PRIu64 ", packet space: %" PRIu64 "\n", exporter->info.id, packetInterval,
+                       spaceInterval);
         } else if (defaultSampler) {
             // static default sampler
             packetInterval = defaultSampler->record.packetInterval;
             spaceInterval = defaultSampler->record.spaceInterval;
             SetFlag(recordHeaderV3->flags, V3_FLAG_SAMPLED);
-            dbg_printf("[%u] Found static default sampler - packet interval: %llu, packet space: %llu\n", exporter->info.id, packetInterval,
-                       spaceInterval);
+            dbg_printf("[%u] Found static default sampler - packet interval: %" PRIu64 ", packet space: %" PRIu64 "\n", exporter->info.id,
+                       packetInterval, spaceInterval);
         }
         intervalTotal = packetInterval + spaceInterval;
 
@@ -1444,14 +1450,18 @@ static void Process_ipfix_data(exporterDomain_t *exporter, uint32_t ExportTime,
                 dbg_printf("Calculate first/last from option SysUpTime\n");
                 genericFlow->msecFirst = exporter->SysUpTime + stack[STACK_MSECFIRST];
                 genericFlow->msecLast = exporter->SysUpTime + stack[STACK_MSECLAST];
+            } else if (stack[STACK_SECFIRST]) {
+                dbg_printf("first/last sec abs.\n");
+                genericFlow->msecFirst = stack[STACK_SECFIRST] * (uint64_t)1000;
+                genericFlow->msecLast = stack[STACK_SECLAST] * (uint64_t)1000;
             }
 
             if (genericFlow->msecFirst < fs->msecFirst) fs->msecFirst = genericFlow->msecFirst;
             if (genericFlow->msecLast > fs->msecLast) fs->msecLast = genericFlow->msecLast;
-            dbg_printf("msecFrist: %llu\n", genericFlow->msecFirst);
-            dbg_printf("msecLast : %llu\n", genericFlow->msecLast);
-            dbg_printf("packets : %llu\n", (long long unsigned)genericFlow->inPackets);
-            dbg_printf("bytes : %llu\n", (long long unsigned)genericFlow->inBytes);
+            dbg_printf("msecFrist: %" PRIu64 "\n", genericFlow->msecFirst);
+            dbg_printf("msecLast : %" PRIu64 "\n", genericFlow->msecLast);
+            dbg_printf("packets : %" PRIu64 "\n", genericFlow->inPackets);
+            dbg_printf("bytes : %" PRIu64 "\n", genericFlow->inBytes);
 
             if (spaceInterval > 0) {
                 genericFlow->inPackets = genericFlow->inPackets * intervalTotal / (uint64_t)packetInterval;
@@ -1594,7 +1604,7 @@ static inline void Process_ipfix_sampler_option_data(exporterDomain_t *exporter,
             dbg_printf("New std sampler: algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.algorithm,
                        sampler_record.packetInterval, sampler_record.spaceInterval);
         } else {
-            dbg_printf("ID : %lld, algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
+            dbg_printf("ID : %" PRId64 ", algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
                        sampler_record.packetInterval, sampler_record.spaceInterval);
         }
 
@@ -1619,7 +1629,7 @@ static inline void Process_ipfix_sampler_option_data(exporterDomain_t *exporter,
                 LogError("Process_ipfix_option: Zero sampling interval -> sampling == 1", __FILE__, __LINE__);
             }
         }
-        dbg_printf("ID : %lld, algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
+        dbg_printf("ID : %" PRId64 ", algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
                    sampler_record.packetInterval, sampler_record.spaceInterval);
 
         InsertSampler(fs, exporter, &sampler_record);
@@ -1646,7 +1656,7 @@ static void Process_ipfix_nbar_option_data(exporterDomain_t *exporter, FlowSourc
     dbg_printf("[%u] nbar option data - records: %u, size: %zu\n", exporter->info.id, numRecords, option_size);
 
     if (numRecords == 0 || option_size == 0 || option_size > size_left) {
-        LogError("Process_nbar_option: nbar option size error: option size: %u, size left: %u", option_size, size_left);
+        LogError("Process_nbar_option: nbar option size error: option size: %zu, size left: %u", option_size, size_left);
         return;
     }
 
@@ -1779,7 +1789,7 @@ static void Process_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_t *
     dbg_printf("[%u] name option data - records: %u, size: %zu\n", exporter->info.id, numRecords, option_size);
 
     if (numRecords == 0 || option_size == 0 || option_size > size_left) {
-        LogError("Process_ifvrf_option: nbar option size error: option size: %u, size left: %u", option_size, size_left);
+        LogError("Process_ifvrf_option: nbar option size error: option size: %zu, size left: %u", option_size, size_left);
         return;
     }
 
@@ -1875,7 +1885,7 @@ static void Process_ipfix_SysUpTime_option_data(exporterDomain_t *exporter, temp
     uint8_t *in = (uint8_t *)(data_flowset + 4);  // skip flowset header
     if (CHECK_OPTION_DATA(size_left, optionTemplate->SysUpOption)) {
         exporter->SysUpTime = Get_val(in, optionTemplate->SysUpOption.offset, optionTemplate->SysUpOption.length);
-        dbg_printf("Extracted SysUpTime : %llu\n", exporter->SysUpTime);
+        dbg_printf("Extracted SysUpTime : %" PRIu64 "\n", exporter->SysUpTime);
     } else {
         LogError("Process_ipfix_option: %s line %d: Not enough data for option data", __FILE__, __LINE__);
         return;
@@ -1944,8 +1954,8 @@ void Process_IPFIX(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
     flowset_header = (void *)ipfix_header + IPFIX_HEADER_LENGTH;
     size_left -= IPFIX_HEADER_LENGTH;
 
-    dbg_printf("\n[%u] process packet: %u, export time: %s, TemplateRecords: %llu, DataRecords: %llu, buffer: %li \n", ObservationDomain, pkg_num++,
-               UNIX2ISO(ExportTime), (long long unsigned)exporter->TemplateRecords, (long long unsigned)exporter->DataRecords, size_left);
+    dbg_printf("\n[%u] process packet: %u, export time: %s, TemplateRecords: %" PRIu64 ", DataRecords: %" PRIu64 ", buffer: %zd \n",
+               ObservationDomain, pkg_num++, UNIX2ISO(ExportTime), exporter->TemplateRecords, exporter->DataRecords, size_left);
     dbg_printf("[%u] Sequence: %u\n", ObservationDomain, Sequence);
 
     // sequence check
