@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2024, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -33,6 +33,7 @@
 
 #include <arpa/inet.h>
 #include <errno.h>
+#include <inttypes.h>
 #include <netinet/in.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -69,6 +70,8 @@ enum {
     STACK_MSECFIRST,
     STACK_MSECLAST,
     STACK_SAMPLER,
+    STACK_SECFIRST,
+    STACK_SECLAST,
     STACK_MSEC,
     STACK_SYSUPTIME,
     STACK_CLIENT_USEC,
@@ -159,6 +162,8 @@ static const struct v9TranslationMap_s {
     {NF9_BGP_V4_NEXT_HOP, SIZEbgp4NextIP, NumberCopy, EXbgpNextHopV4ID, OFFbgp4NextIP, STACK_NONE, "IPv4 bgp next hop"},
     {NF9_LAST_SWITCHED, Stack_ONLY, NumberCopy, EXgenericFlowID, OFFmsecLast, STACK_MSECLAST, "msec last SysupTime"},
     {NF9_FIRST_SWITCHED, Stack_ONLY, NumberCopy, EXgenericFlowID, OFFmsecFirst, STACK_MSECFIRST, "msec first SysupTime"},
+    {NF_F_flowStartSeconds, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SECFIRST, "sec first seen"},
+    {NF_F_flowEndSeconds, Stack_ONLY, NumberCopy, EXnull, 0, STACK_SECLAST, "sec last seen"},
     {NF9_OUT_BYTES, SIZEoutBytes, NumberCopy, EXcntFlowID, OFFoutBytes, STACK_NONE, "output bytes delta counter"},
     {NF9_OUT_PKTS, SIZEoutPackets, NumberCopy, EXcntFlowID, OFFoutPackets, STACK_NONE, "output packet delta counter"},
     {NF9_IPV6_SRC_ADDR, SIZEsrc6Addr, NumberCopy, EXipv6FlowID, OFFsrc6Addr, STACK_NONE, "IPv6 src addr"},
@@ -166,9 +171,13 @@ static const struct v9TranslationMap_s {
     {NF9_IPV6_SRC_MASK, SIZEsrcMask, NumberCopy, EXflowMiscID, OFFsrcMask, STACK_NONE, "src mask bits"},
     {NF9_IPV6_DST_MASK, SIZEdstMask, NumberCopy, EXflowMiscID, OFFdstMask, STACK_NONE, "dst mask bits"},
     {NF9_ICMP, Stack_ONLY, NumberCopy, EXgenericFlowID, OFFdstPort, STACK_ICMP, "icmp type/code"},
+    {NF9_MIN_TTL, SIZEminTTL, NumberCopy, EXipInfoID, OFFminTTL, STACK_NONE, "flow min TTL"},
+    {NF9_MAX_TTL, SIZEmaxTTL, NumberCopy, EXipInfoID, OFFmaxTTL, STACK_NONE, "flow max TTL"},
     {NF9_DST_TOS, SIZEdstTos, NumberCopy, EXflowMiscID, OFFdstTos, STACK_NONE, "post IP class of Service"},
+    {NF9_MIN_TTL, SIZEminTTL, NumberCopy, EXipInfoID, OFFminTTL, STACK_NONE, "flow min TTL"},
+    {NF9_MAX_TTL, SIZEmaxTTL, NumberCopy, EXipInfoID, OFFmaxTTL, STACK_NONE, "flow max TTL"},
     {NF_F_flowEndReason, SIZEflowEndReason, NumberCopy, EXflowMiscID, OFFflowEndReason, STACK_NONE, "Flow end reason"},
-    {NF_F_ipTTL, SIZEipTTL, NumberCopy, EXipInfoID, OFFipTTL, STACK_NONE, "IP ttl"},
+    {NF_F_ipTTL, SIZEminTTL, NumberCopy, EXipInfoID, OFFminTTL, STACK_NONE, "flow min TTL"},
     {NF_F_fragmentFlags, SIZEfragmentFlags, NumberCopy, EXipInfoID, OFFfragmentFlags, STACK_NONE, "IP fragment flags"},
     {NF9_IN_SRC_MAC, SIZEinSrcMac, NumberCopy, EXmacAddrID, OFFinSrcMac, STACK_NONE, "in src MAC addr"},
     {NF9_OUT_DST_MAC, SIZEoutDstMac, NumberCopy, EXmacAddrID, OFFoutDstMac, STACK_NONE, "out dst MAC addr"},
@@ -454,7 +463,7 @@ static inline exporterDomain_t *getExporter(FlowSource_t *fs, uint32_t exporter_
 static void InsertSampler(FlowSource_t *fs, exporterDomain_t *exporter, sampler_record_t *sampler_record) {
     sampler_t *sampler;
 
-    dbg_printf("[%u] Insert Sampler: Exporter is 0x%llu\n", exporter->info.id, (long long unsigned)exporter);
+    dbg_printf("[%u] Insert Sampler: Exporter is 0x%p\n", exporter->info.id, (void *)exporter);
     if (!exporter->sampler) {
         // no samplers so far
         sampler = (sampler_t *)malloc(sizeof(sampler_t));
@@ -921,7 +930,7 @@ static inline void Process_v9_option_templates(exporterDomain_t *exporter, void
     }
     optionTemplate->optionSize = offset;
 
-    dbg_printf("\n[%u] Option size: %llu, flags: %llx\n", exporter->info.id, optionTemplate->optionSize, optionTemplate->flags);
+    dbg_printf("\n[%u] Option size: %" PRIu64 ", flags: %" PRIx64 "\n", exporter->info.id, optionTemplate->optionSize, optionTemplate->flags);
     if (optionTemplate->flags) {
         // if it exitsts - remove old template on exporter with same ID
         templateList_t *template = newTemplate(exporter, tableID);
@@ -1034,8 +1043,8 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
         // map file record to output buffer
         outBuff = GetCurrentCursor(fs->dataBlock);
 
-        dbg_printf("[%u] Process data record: %u addr: %llu, size_left: %u buff_avail: %u\n", exporter->info.id, processed_records,
-                   (long long unsigned)((ptrdiff_t)inBuff - (ptrdiff_t)data_flowset), size_left, buffAvail);
+        dbg_printf("[%u] Process data record: %u addr: %p, size_left: %u buff_avail: %u\n", exporter->info.id, processed_records,
+                   (void *)((ptrdiff_t)inBuff - (ptrdiff_t)data_flowset), size_left, buffAvail);
 
         // process record
         AddV3Header(outBuff, recordHeaderV3);
@@ -1081,7 +1090,7 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
                 break;
         }
 
-        dbg_printf("New record added with %u elements and size: %u, sequencer inLength: %lu, outLength: %lu\n", recordHeaderV3->numElements,
+        dbg_printf("New record added with %u elements and size: %u, sequencer inLength: %zu, outLength: %zu\n", recordHeaderV3->numElements,
                    recordHeaderV3->size, sequencer->inLength, sequencer->outLength);
 
         // add router IP
@@ -1143,27 +1152,29 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
             packetInterval = overwriteSampler->record.packetInterval;
             spaceInterval = overwriteSampler->record.spaceInterval;
             SetFlag(recordHeaderV3->flags, V3_FLAG_SAMPLED);
-            dbg_printf("[%u] Overwrite sampling - packet interval: %llu, packet space: %llu\n", exporter->info.id, packetInterval, spaceInterval);
+            dbg_printf("[%u] Overwrite sampling - packet interval: %" PRIu64 ", packet space: %" PRIu64 "\n", exporter->info.id, packetInterval,
+                       spaceInterval);
         } else if (sampler) {
             // individual assigned sampler ID
             packetInterval = sampler->record.packetInterval;
             spaceInterval = sampler->record.spaceInterval;
             SetFlag(recordHeaderV3->flags, V3_FLAG_SAMPLED);
-            dbg_printf("[%u] Found assigned sampler ID %u - packet interval: %llu, packet space: %llu\n", exporter->info.id, sampler_id,
+            dbg_printf("[%u] Found assigned sampler ID %u - packet interval: %" PRIu64 ", packet space: %" PRIu64 "\n", exporter->info.id, sampler_id,
                        packetInterval, spaceInterval);
         } else if (genericSampler) {
             // global sampler ID
             packetInterval = genericSampler->record.packetInterval;
             spaceInterval = genericSampler->record.spaceInterval;
             SetFlag(recordHeaderV3->flags, V3_FLAG_SAMPLED);
-            dbg_printf("[%u] Found generic sampler - packet interval: %llu, packet space: %llu\n", exporter->info.id, packetInterval, spaceInterval);
+            dbg_printf("[%u] Found generic sampler - packet interval: %" PRIu64 ", packet space: %" PRIu64 "\n", exporter->info.id, packetInterval,
+                       spaceInterval);
         } else if (defaultSampler) {
             // static default sampler
             packetInterval = defaultSampler->record.packetInterval;
             spaceInterval = defaultSampler->record.spaceInterval;
             SetFlag(recordHeaderV3->flags, V3_FLAG_SAMPLED);
-            dbg_printf("[%u] Found static default sampler - packet interval: %llu, packet space: %llu\n", exporter->info.id, packetInterval,
-                       spaceInterval);
+            dbg_printf("[%u] Found static default sampler - packet interval: %" PRIu64 ", packet space: %" PRIu64 "\n", exporter->info.id,
+                       packetInterval, spaceInterval);
         }
         intervalTotal = packetInterval + spaceInterval;
 
@@ -1186,14 +1197,17 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
 
                 // end time in msecs
                 genericFlow->msecLast = (uint64_t)Last + exporter->boot_time;
+            } else if (stack[STACK_SECFIRST]) {
+                genericFlow->msecFirst = stack[STACK_SECFIRST] * (uint64_t)1000;
+                genericFlow->msecLast = stack[STACK_SECLAST] * (uint64_t)1000;
             }
 
             if (genericFlow->msecFirst < fs->msecFirst) fs->msecFirst = genericFlow->msecFirst;
             if (genericFlow->msecLast > fs->msecLast) fs->msecLast = genericFlow->msecLast;
-            dbg_printf("msecFrist: %llu\n", (long long unsigned)genericFlow->msecFirst);
-            dbg_printf("msecLast : %llu\n", (long long unsigned)genericFlow->msecLast);
-            dbg_printf("packets : %llu\n", (long long unsigned)genericFlow->inPackets);
-            dbg_printf("bytes : %llu\n", (long long unsigned)genericFlow->inBytes);
+            dbg_printf("msecFrist: %" PRIu64 "\n", genericFlow->msecFirst);
+            dbg_printf("msecLast : %" PRIu64 "\n", genericFlow->msecLast);
+            dbg_printf("packets : %" PRIu64 "\n", genericFlow->inPackets);
+            dbg_printf("bytes : %" PRIu64 "\n", genericFlow->inBytes);
 
             if (spaceInterval > 0) {
                 genericFlow->inPackets = genericFlow->inPackets * intervalTotal / (uint64_t)packetInterval;
@@ -1263,21 +1277,41 @@ static inline void Process_v9_data(exporterDomain_t *exporter, void *data_flowse
         EXnselCommon_t *nselCommon = sequencer->offsetCache[EXnselCommonID];
         if (nselCommon) {
             nselCommon->msecEvent = stack[STACK_MSEC];
-            if (genericFlow) {
-                genericFlow->msecFirst = stack[STACK_MSEC];
-                genericFlow->msecLast = stack[STACK_MSEC];
+            if (nselCommon->msecEvent) {
+                if (genericFlow) {
+                    dbg_printf("Copy nsel Event time: %" PRIu64 " overwriting %" PRIu64 "\n", nselCommon->msecEvent, genericFlow->msecFirst);
+                    genericFlow->msecFirst = stack[STACK_MSEC];
+                    genericFlow->msecLast = stack[STACK_MSEC];
+                }
+            } else {
+                if (genericFlow) {
+                    dbg_printf("Copy msecFirst to nsel Event time: %" PRIu64 "\n", genericFlow->msecFirst);
+                    nselCommon->msecEvent = genericFlow->msecFirst;
+                }
             }
             SetFlag(recordHeaderV3->flags, V3_FLAG_EVENT);
+            dbg_printf("Nsel event time: %" PRIu64 "\n", nselCommon->msecEvent);
         }
         EXnatCommon_t *natCommon = sequencer->offsetCache[EXnatCommonID];
         if (natCommon) {
             natCommon->msecEvent = stack[STACK_MSEC];
-            if (genericFlow) {
-                genericFlow->msecFirst = stack[STACK_MSEC];
-                genericFlow->msecLast = stack[STACK_MSEC];
+            if (natCommon->msecEvent) {
+                if (genericFlow) {
+                    dbg_printf("Copy nat Event time: %" PRIu64 " overwriting %" PRIu64 "\n", natCommon->msecEvent, genericFlow->msecFirst);
+                    genericFlow->msecFirst = stack[STACK_MSEC];
+                    genericFlow->msecLast = stack[STACK_MSEC];
+                }
+            } else {
+                if (genericFlow) {
+                    dbg_printf("Copy msecFirst to nat Event time: %" PRIu64 "\n", genericFlow->msecFirst);
+                    natCommon->msecEvent = genericFlow->msecFirst;
+                }
             }
             SetFlag(recordHeaderV3->flags, V3_FLAG_EVENT);
+            dbg_printf("Nat event time: %" PRIu64 "\n", natCommon->msecEvent);
         }
+        dbg_printf("Final msecFrist: %" PRIu64 "\n", genericFlow->msecFirst);
+        dbg_printf("Final msecLast : %" PRIu64 "\n", genericFlow->msecLast);
 
         // nprobe latency
         EXlatency_t *latency = sequencer->offsetCache[EXlatencyID];
@@ -1355,7 +1389,7 @@ static inline void Process_v9_sampler_option_data(exporterDomain_t *exporter, Fl
             dbg_printf("New std sampler: algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.algorithm,
                        sampler_record.packetInterval, sampler_record.spaceInterval);
         } else {
-            dbg_printf("ID : %lld, algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
+            dbg_printf("ID : %" PRId64 ", algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
                        sampler_record.packetInterval, sampler_record.spaceInterval);
         }
 
@@ -1380,7 +1414,7 @@ static inline void Process_v9_sampler_option_data(exporterDomain_t *exporter, Fl
                 LogError("Process_v9_option: Zero sampling interval -> sampling == 1", __FILE__, __LINE__);
             }
         }
-        dbg_printf("ID : %lld, algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
+        dbg_printf("ID : %" PRId64 ", algorithm : %u, packet interval: %u, packet space: %u\n", sampler_record.id, sampler_record.algorithm,
                    sampler_record.packetInterval, sampler_record.spaceInterval);
 
         InsertSampler(fs, exporter, &sampler_record);
@@ -1407,7 +1441,7 @@ static void Process_v9_nbar_option_data(exporterDomain_t *exporter, FlowSource_t
     dbg_printf("[%u] nbar option data - records: %u, size: %zu\n", exporter->info.id, numRecords, option_size);
 
     if (numRecords == 0 || option_size == 0 || option_size > size_left) {
-        LogError("Process_nbar_option: nbar option size error: option size: %u, size left: %u", option_size, size_left);
+        LogError("Process_nbar_option: nbar option size error: option size: %zu, size left: %u", option_size, size_left);
         return;
     }
 
@@ -1538,7 +1572,7 @@ static void Process_v9_ifvrf_option_data(exporterDomain_t *exporter, FlowSource_
     dbg_printf("[%u] name option data - records: %u, size: %zu\n", exporter->info.id, numRecords, option_size);
 
     if (numRecords == 0 || option_size == 0 || option_size > size_left) {
-        LogError("Process_ifvrf_option: nbar option size error: option size: %u, size left: %u", option_size, size_left);
+        LogError("Process_ifvrf_option: nbar option size error: option size: %zu, size left: %u", option_size, size_left);
         return;
     }
 
@@ -1635,7 +1669,7 @@ static void Process_v9_SysUpTime_option_data(exporterDomain_t *exporter, templat
     uint8_t *in = (uint8_t *)(data_flowset + 4);  // skip flowset header
     if (CHECK_OPTION_DATA(size_left, optionTemplate->SysUpOption)) {
         exporter->SysUpTime = Get_val(in, optionTemplate->SysUpOption.offset, optionTemplate->SysUpOption.length);
-        dbg_printf("Extracted SysUpTime : %llu\n", exporter->SysUpTime);
+        dbg_printf("Extracted SysUpTime : %" PRIu64 "\n", exporter->SysUpTime);
     } else {
         LogError("Process_v9_option: %s line %d: Not enough data for option data", __FILE__, __LINE__);
         return;
@@ -1709,7 +1743,7 @@ void Process_v9(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
 
 #ifdef DEVEL
     uint32_t expected_records = ntohs(v9_header->count);
-    printf("[%u] records: %u, buffer: %li \n", exporter_id, expected_records, size_left);
+    printf("[%u] records: %u, buffer: %zd \n", exporter_id, expected_records, size_left);
     printf("SourceID: %u, Sysuptime: %u.%u\n", v9_header->source_id, v9_header->SysUptime, v9_header->unix_secs);
 #endif
 
@@ -1733,7 +1767,7 @@ void Process_v9(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
                        (long long)exporter->sequence, (long long)distance);
         }
     }
-    dbg_printf("Sequence: %llu\n", exporter->sequence);
+    dbg_printf("Sequence: %" PRIu64 "\n", exporter->sequence);
 
     processed_records = 0;
 
