@@ -185,8 +185,8 @@ typedef struct common_header_s {
 #define SAMPLING_PACKET_INTERVAL 305
 #define SAMPLING_SPACE_INTERVAL 306
 
-// #define NF9_MIN_TTL			52
-// #define NF9_MAX_TTL			53
+#define NF9_MIN_TTL 52
+#define NF9_MAX_TTL 53
 // #define NF9_IPV4_IDENT		54
 
 #define NF9_DST_TOS 55
@@ -226,6 +226,8 @@ typedef struct common_header_s {
 // IPFIX elements in v9 ( # > 127 )
 #define NF_F_BGP_ADJ_NEXT_AS 128
 #define NF_F_BGP_ADJ_PREV_AS 129
+#define NF_F_flowStartSeconds 150
+#define NF_F_flowEndSeconds 151
 #define NF_F_flowEndReason 136
 #define NF_F_dot1qVlanId 243
 #define NF_F_postDot1qVlanId 254
