@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2024, Peter Haag
+ *  Copyright (c) 2025, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -97,7 +97,7 @@ static inline exporter_nfd_t *getExporter(FlowSource_t *fs, nfd_header_t *header
     // nothing found
     *e = (exporter_nfd_t *)malloc(sizeof(exporter_nfd_t));
     if (!(*e)) {
-        LogError("Process_nfd: malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("Process_nfd: malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return NULL;
     }
     memset((void *)(*e), 0, sizeof(exporter_nfd_t));
@@ -179,7 +179,7 @@ void Process_nfd(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
 
     uint16_t count = ntohl(pcapd_header->numRecord);
     uint32_t numRecords = 0;
-    dbg_printf("Process nfd packet: %llu, size: %zd, recordCnt: %u\n", exporter->packets, in_buff_cnt, count);
+    dbg_printf("Process nfd packet: %" PRIu64 ", size: %zd, recordCnt: %u\n", exporter->packets, in_buff_cnt, count);
 
     if ((sizeof(nfd_header_t) + sizeof(recordHeaderV3_t)) > size_left) {
         LogError("Process_nfd: Not enough data.");
@@ -199,7 +199,7 @@ void Process_nfd(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
         }
 
         if (recordHeaderV3->size > size_left) {
-            LogError("Process_nfd: record size error. Size v3header: %u > size left: %d", recordHeaderV3->size, size_left);
+            LogError("Process_nfd: record size error. Size v3header: %u > size left: %zd", recordHeaderV3->size, size_left);
             LogError("Process_nfd: expected %u records, processd: %u", count, numRecords);
             return;
         }
