@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2012-2024, Peter Haag
+ *  Copyright (c) 2012-2025, Peter Haag
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -46,20 +46,18 @@
 #include "nfxV3.h"
 #include "util.h"
 
-/* global */
-static exporter_t **exporter_list = NULL;
-
-/* local variables */
+// exporter local variables
 #define MAX_EXPORTERS 65536
 
+static exporter_t **exporter_list = NULL;
+static uint32_t nextFree = 1;
+
 static const struct versionString_s {
     uint16_t version;
     char *string;
 } versionString[] = {{5, "netflow v5"}, {9, "netflow v9"}, {10, "ipfix v10"}, {9999, "sflow"}, {0, NULL}};
 
 /* local prototypes */
-static exporter_t *exporter_root;
-
 static char *getVersionString(uint16_t nfversion);
 
 #include "nffile_inline.c"
@@ -76,17 +74,16 @@ static char *getVersionString(uint16_t nfversion) {
 int InitExporterList(void) {
     exporter_list = (exporter_t **)calloc(MAX_EXPORTERS, sizeof(exporter_t *));
     if (!exporter_list) {
-        LogError("calloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("calloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return 0;
     }
-    exporter_root = NULL;
     return 1;
 
 }  // End of InitExporterList
 
 int AddExporterInfo(exporter_info_record_t *exporter_record) {
     if (exporter_record->header.size != sizeof(exporter_info_record_t)) {
-        LogError("Corrupt exporter record in %s line %d\n", __FILE__, __LINE__);
+        LogError("Corrupt exporter record in %s line %d", __FILE__, __LINE__);
         return 0;
     }
 
@@ -95,7 +92,7 @@ int AddExporterInfo(exporter_info_record_t *exporter_record) {
     // sanity check
     uint32_t id = exporter_record->sysid;
     if (id >= MAX_EXPORTERS) {
-        LogError("Corrupt exporter record in %s line %d\n", __FILE__, __LINE__);
+        LogError("Corrupt exporter record in %s line %d", __FILE__, __LINE__);
         return 0;
     }
     if (exporter_list[id] != NULL) {
@@ -106,38 +103,37 @@ int AddExporterInfo(exporter_info_record_t *exporter_record) {
             // we are done
             return 2;
         } else {
-            // exporters not identical - move current slot
-            int i;
-            // search first empty slot at the top of the list
-            for (i = id + 1; i < MAX_EXPORTERS && exporter_list[i] != NULL; i++) {
-                ;
-            }
-            if (i >= MAX_EXPORTERS) {
-                // all slots taken
-                LogError("Too many exporters (>256)\n");
+            // exporters not identical - move current slot to next free slot
+            if (nextFree >= MAX_EXPORTERS) {
+                // all slots taken - no free slot
+                LogError("Too many exporters (>%d)\n", MAX_EXPORTERS);
                 return 0;
             }
-            dbg_printf("Move existing exporter from slot %u, to %i\n", id, i);
-            // else - move slot
-            exporter_list[i] = exporter_list[id];
+            // nextFree slot is guaranteed to be free (NULL)
+            dbg_printf("Move existing exporter from slot %u, to nextFree %i\n", id, nextFree);
+            exporter_list[nextFree] = exporter_list[id];
+            exporter_list[nextFree]->info.sysid = nextFree;
             exporter_list[id] = NULL;
-            exporter_record->sysid = i;
+            exporter_record->sysid = id;
+
+            // advance next free - nextFree could reach MAX_EXPORTERS
+            nextFree++;
         }
     }
 
     // slot[id] is now available
     exporter_list[id] = (exporter_t *)calloc(1, sizeof(exporter_t));
     if (!exporter_list[id]) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return 0;
     }
 
-    // SPARC gcc fails here, if we use directly a pointer to the struct.
-    // SPARC barfs and core dumps otherwise
-    // memcpy((void *)&(exporter_list[id]->info), (void *)exporter_record, sizeof(exporter_info_record_t));
+    memcpy((void *)&(exporter_list[id]->info), (void *)exporter_record, sizeof(exporter_info_record_t));
+    /*
     char *p1 = (char *)&(exporter_list[id]->info);
     char *p2 = (char *)exporter_record;
     for (int i = 0; i < sizeof(exporter_info_record_t); i++) *p1++ = *p2++;
+    */
 
     dbg_printf("Insert exporter record in Slot: %i, Sysid: %u\n", id, exporter_record->sysid);
 
@@ -162,44 +158,41 @@ int AddExporterInfo(exporter_info_record_t *exporter_record) {
             printf("**** Exporter IP version unknown ****\n");
         }
     }
-    printf("\n");
 #endif
 
-    if (!exporter_root) {
-        exporter_root = exporter_list[id];
-    }
-
+    while (nextFree < MAX_EXPORTERS && exporter_list[nextFree] != NULL) nextFree++;
+    dbg_printf("NextFree slot is %d\n\n", nextFree);
     return 1;
+
 }  // End of AddExporterInfo
 
-int AddSamplerLegacyRecord(samplerV0_record_t *sampler_record) {
-    if (sampler_record->size != sizeof(samplerV0_record_t)) {
-        LogError("Corrupt sampler record in %s line %d", __FILE__, __LINE__);
-        return 0;
+sampler_record_t *ConvertLegacyRecord(samplerV0_record_t *legacy_record) {
+    if (legacy_record->size != sizeof(samplerV0_record_t)) {
+        LogError("Corrupt legacy sampler record detected in %s line %d", __FILE__, __LINE__);
+        return NULL;
     }
 
-    sampler_record_t convert_record = {0};
-    sampler_record_t *record = &convert_record;
-    if (sampler_record->size == sizeof(samplerV0_record_t)) {
-        samplerV0_record_t *samplerV0_record = (samplerV0_record_t *)sampler_record;
-
-        convert_record.type = SamplerRecordType;
-        convert_record.size = sizeof(sampler_record_t);
-        convert_record.exporter_sysid = samplerV0_record->exporter_sysid;
-        convert_record.id = samplerV0_record->id;
-        convert_record.algorithm = samplerV0_record->algorithm;
-        convert_record.packetInterval = 1;
-        convert_record.spaceInterval = samplerV0_record->interval - 1;
-        record = &convert_record;
+    sampler_record_t *sampler_record = calloc(1, sizeof(sampler_record_t));
+    if (!sampler_record) {
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
+        return NULL;
     }
-    return AddSamplerRecord(record);
 
-}  // End of AddSamplerLegacyRecord
+    sampler_record->type = SamplerRecordType;
+    sampler_record->size = sizeof(sampler_record_t);
+    sampler_record->exporter_sysid = legacy_record->exporter_sysid;
+    sampler_record->id = legacy_record->id;
+    sampler_record->algorithm = legacy_record->algorithm;
+    sampler_record->packetInterval = 1;
+    sampler_record->spaceInterval = legacy_record->interval - 1;
+
+    return sampler_record;
+}  // End of ConvertLegacyRecord
 
 int AddSamplerRecord(sampler_record_t *sampler_record) {
     uint32_t id = sampler_record->exporter_sysid;
     if (id >= MAX_EXPORTERS) {
-        LogError("Corrupt sampler record in %s line %d\n", __FILE__, __LINE__);
+        LogError("Corrupt sampler record in %s line %d", __FILE__, __LINE__);
         return 0;
     }
 
@@ -223,7 +216,7 @@ int AddSamplerRecord(sampler_record_t *sampler_record) {
 
     *sampler = (sampler_t *)malloc(sizeof(sampler_t));
     if (!*sampler) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return 0;
     }
     (*sampler)->next = NULL;
@@ -241,6 +234,7 @@ int AddSamplerRecord(sampler_record_t *sampler_record) {
             printf("Exporter SysID: %u,	Sampler: algorithm: %u, packet interval: %u, packet space: %u\n", sampler_record->exporter_sysid,
                    sampler_record->algorithm, sampler_record->packetInterval, sampler_record->spaceInterval);
         }
+        printf("\n");
     }
 #endif
 
@@ -249,13 +243,13 @@ int AddSamplerRecord(sampler_record_t *sampler_record) {
 
 int AddExporterStat(exporter_stats_record_t *stat_record) {
     if (stat_record->header.size < sizeof(exporter_stats_record_t)) {
-        LogError("Corrupt exporter record in %s line %d\n", __FILE__, __LINE__);
+        LogError("Corrupt exporter record in %s line %d", __FILE__, __LINE__);
         return 0;
     }
 
     size_t required = sizeof(exporter_stats_record_t) + (stat_record->stat_count - 1) * sizeof(struct exporter_stat_s);
     if ((stat_record->stat_count == 0) || (stat_record->header.size != required)) {
-        LogError("Corrupt exporter record in %s line %d\n", __FILE__, __LINE__);
+        LogError("Corrupt exporter record in %s line %d", __FILE__, __LINE__);
         return 0;
     }
 
@@ -265,7 +259,7 @@ int AddExporterStat(exporter_stats_record_t *stat_record) {
     if (((ptrdiff_t)stat_record & 0x7) != 0) {
         rec = (exporter_stats_record_t *)malloc(stat_record->header.size);
         if (!rec) {
-            LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             exit(255);
         }
         memcpy(rec, stat_record, stat_record->header.size);
@@ -279,18 +273,18 @@ int AddExporterStat(exporter_stats_record_t *stat_record) {
     for (int i = 0; i < rec->stat_count; i++) {
         uint32_t id = rec->stat[i].sysid;
         if (id >= MAX_EXPORTERS) {
-            LogError("Corrupt exporter record in %s line %d\n", __FILE__, __LINE__);
+            LogError("Corrupt exporter record in %s line %d", __FILE__, __LINE__);
             return 0;
         }
         if (!exporter_list[id]) {
-            LogError("Exporter SysID: %u not found! - Skip stat record record.\n");
+            LogError("Exporter SysID: %u not found! - Skip stat record record", id);
             continue;
         }
         exporter_list[id]->sequence_failure += rec->stat[i].sequence_failure;
         exporter_list[id]->packets += rec->stat[i].packets;
         exporter_list[id]->flows += rec->stat[i].flows;
-        dbg_printf("Update exporter stat for SysID: %i: Sequence failures: %u, packets: %llu, flows: %llu\n", id, exporter_list[id]->sequence_failure,
-                   exporter_list[id]->packets, exporter_list[id]->flows);
+        dbg_printf("Update exporter stat for SysID: %i: Sequence failures: %u, packets: %" PRIu64 ", flows: %" PRIu64 "\n", id,
+                   exporter_list[id]->sequence_failure, exporter_list[id]->packets, exporter_list[id]->flows);
     }
 
     if (use_copy) free(rec);
@@ -301,7 +295,7 @@ int AddExporterStat(exporter_stats_record_t *stat_record) {
 
 exporter_t *GetExporterInfo(int exporterID) {
     if (exporterID >= MAX_EXPORTERS) {
-        LogError("Corrupt exporter record in %s line %d\n", __FILE__, __LINE__);
+        LogError("Corrupt exporter record in %s line %d", __FILE__, __LINE__);
         return NULL;
     }
 
@@ -316,14 +310,13 @@ dataBlock_t *ExportExporterList(nffile_t *nffile, dataBlock_t *dataBlock) {
 
     int i = 1;
     while (i < MAX_EXPORTERS && exporter_list[i] != NULL) {
-        exporter_info_record_t *exporter;
-        sampler_t *sampler;
-
-        exporter = &exporter_list[i]->info;
+        exporter_info_record_t *exporter = &exporter_list[i]->info;
+        dbg_printf("Dump exporter: %d\n", i);
         dataBlock = AppendToBuffer(nffile, dataBlock, (void *)exporter, exporter->header.size);
 
-        sampler = exporter_list[i]->sampler;
+        sampler_t *sampler = exporter_list[i]->sampler;
         while (sampler) {
+            dbg_printf("  Dump sampler for exporter: %d\n", i);
             dataBlock = AppendToBuffer(nffile, dataBlock, (void *)&(sampler->record), sampler->record.size);
             sampler = sampler->next;
         }
@@ -379,11 +372,15 @@ void PrintExporters(void) {
                         LogError("Failed to add sampler record\n");
                     }
                     break;
-                case SamplerLegacyRecordType:
-                    if (!AddSamplerLegacyRecord((samplerV0_record_t *)record)) {
-                        LogError("Failed to add legacy sampler record\n");
+                case SamplerLegacyRecordType: {
+                    sampler_record_t *sampler_record = ConvertLegacyRecord((samplerV0_record_t *)record);
+                    if (sampler_record != NULL) {
+                        if (!AddSamplerRecord(sampler_record)) {
+                            LogError("Failed to add sampler record\n");
+                        }
+                        free(sampler_record);
                     }
-                    break;
+                } break;
             }
             // Advance pointer by number of bytes for netflow record
             record = (record_header_t *)((void *)record + record->size);
@@ -414,9 +411,9 @@ void PrintExporters(void) {
             uint32_t _ip = htonl(exporter->ip.V4);
             inet_ntop(AF_INET, &_ip, ipstr, sizeof(ipstr));
             if (exporter_list[i]->flows)
-                printf("SysID: %u, IP: %16s, version: %s, ID: %2u, Sequence failures: %u, packets: %llu, flows: %llu\n", exporter->sysid, ipstr,
-                       getVersionString(exporter->version), exporter->id, exporter_list[i]->sequence_failure,
-                       (long long unsigned)exporter_list[i]->packets, (long long unsigned)exporter_list[i]->flows);
+                printf("SysID: %u, IP: %16s, version: %s, ID: %2u, Sequence failures: %u, packets: %" PRIu64 ", flows: %" PRIu64 "\n",
+                       exporter->sysid, ipstr, getVersionString(exporter->version), exporter->id, exporter_list[i]->sequence_failure,
+                       exporter_list[i]->packets, exporter_list[i]->flows);
             else
                 printf("SysID: %u, IP: %16s, version: %s, ID: %2u - no flows sent\n", exporter->sysid, ipstr, getVersionString(exporter->version),
                        exporter->id);
@@ -425,9 +422,9 @@ void PrintExporters(void) {
             uint64_t _ip[2] = {htonll(exporter->ip.V6[0]), htonll(exporter->ip.V6[1])};
             inet_ntop(AF_INET6, &_ip, ipstr, sizeof(ipstr));
             if (exporter_list[i]->flows)
-                printf("SysID: %u, IP: %40s, version: %s, ID: %2u, Sequence failures: %u, packets: %llu, flows: %llu\n ", exporter->sysid, ipstr,
-                       getVersionString(exporter->version), exporter->id, exporter_list[i]->sequence_failure,
-                       (long long unsigned)exporter_list[i]->packets, (long long unsigned)exporter_list[i]->flows);
+                printf("SysID: %u, IP: %40s, version: %s, ID: %2u, Sequence failures: %u, packets: %" PRIu64 ", flows: %" PRIu64 "\n ",
+                       exporter->sysid, ipstr, getVersionString(exporter->version), exporter->id, exporter_list[i]->sequence_failure,
+                       exporter_list[i]->packets, exporter_list[i]->flows);
             else
                 printf("SysID: %u, IP: %40s, version: %s, ID: %2u\n ", exporter->sysid, ipstr, getVersionString(exporter->version), exporter->id);
         } else {
