@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2024, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -78,6 +78,7 @@
 extern char *FilterFilename;
 
 #define MAXANONWORKERS 8
+#define MAX_FILTER_THREADS 32
 
 typedef struct dataHandle_s {
     dataBlock_t *dataBlock;
@@ -93,14 +94,14 @@ typedef struct prepareArgs_s {
 
 typedef struct filterArgs_s {
     _Atomic int self;
+    _Atomic uint64_t processedRecords;
+    _Atomic uint64_t passedRecords;
+    int hasGeoDB;
     int numWorkers;
     void *engine;
     timeWindow_t *timeWindow;
-    int hasGeoDB;
     queue_t *prepareQueue;
     queue_t *processQueue;
-    _Atomic uint64_t processedRecords;
-    _Atomic uint64_t passedRecords;
 } filterArgs_t;
 
 typedef struct filterStat_s {
@@ -112,7 +113,7 @@ static uint64_t total_bytes = 0;
 static uint64_t totalRecords = 0;
 static uint64_t totalPassed = 0;
 static uint32_t skippedBlocks = 0;
-static uint64_t t_first_flow = 0, t_last_flow = 0;
+static uint64_t t_firstMsec = 0, t_lastMsec = 0;
 static _Atomic uint32_t abortProcessing = 0;
 
 enum processType { FLOWSTAT = 1, ELEMENTSTAT, ELEMENTFLOWSTAT, SORTRECORDS, WRITEFILE, PRINTRECORD };
@@ -189,7 +190,10 @@ static void usage(char *name) {
         "\t\t json     json output format.\n"
         "\t\t ndjson   ndjson log output format (one json object per line).\n"
         "\t\t null     no flow records, only statistics output.\n"
-        "\t\t\tmode may be extended by '6' for full IPv6 listing. e.g.long6, extended6.\n"
+        "\t\t fmt:...  user selected line output format. See nfdump(1)\n"
+        "\t\t csv:...  user selected csv output format. See nfdump(1)\n"
+        "\t\tmode may be extended by '6' for full IPv6 listing. e.g.long6, extended6.\n"
+        "-6\t\tPrint full length of IPv6 addresses in fmt output instead of condensed.\n"
         "-E <file>\tPrint exporter and sampling info for collected flows.\n"
         "-v <file>\tverify netflow data file. Print version and blocks.\n"
         "-W <num>\tOptionally set the number of workers to compress flows\n"
@@ -280,8 +284,8 @@ __attribute__((noreturn)) static void *prepareThread(void *arg) {
         dbg_printf("prepareThread exit\n");
         pthread_exit(NULL);
     }
-    t_first_flow = nffile->stat_record->firstseen;
-    t_last_flow = nffile->stat_record->lastseen;
+    t_firstMsec = nffile->stat_record->firstseen;
+    t_lastMsec = nffile->stat_record->lastseen;
 
     dataHandle_t *dataHandle = NULL;
     uint64_t recordCnt = 0;
@@ -292,7 +296,7 @@ __attribute__((noreturn)) static void *prepareThread(void *arg) {
     while (!done) {
         if (dataHandle == NULL) {
             dataHandle = calloc(1, sizeof(dataHandle_t));
-            dataHandle->ident = nffile->ident;
+            dataHandle->ident = nffile->ident != NULL ? strdup(nffile->ident) : NULL;
         }
         dataHandle->dataBlock = ReadBlock(nffile, NULL);
 
@@ -302,8 +306,10 @@ __attribute__((noreturn)) static void *prepareThread(void *arg) {
             if (GetNextFile(nffile) == NULL) {
                 done = 1;
             } else {
-                if (nffile->stat_record->firstseen < t_first_flow) t_first_flow = nffile->stat_record->firstseen;
-                if (nffile->stat_record->lastseen > t_last_flow) t_last_flow = nffile->stat_record->lastseen;
+                if (nffile->stat_record->firstseen < t_firstMsec) t_firstMsec = nffile->stat_record->firstseen;
+                if (nffile->stat_record->lastseen > t_lastMsec) t_lastMsec = nffile->stat_record->lastseen;
+                if (dataHandle->ident) free(dataHandle->ident);
+                dataHandle->ident = nffile->ident != NULL ? strdup(nffile->ident) : NULL;
             }
             continue;
         }
@@ -376,16 +382,16 @@ __attribute__((noreturn)) static void *filterThread(void *arg) {
     uint64_t twin_msecFirst, twin_msecLast;
     twin_msecFirst = twin_msecLast = 0;
     if (timeWindow) {
-        twin_msecFirst = timeWindow->first * 1000LL;
-        if (timeWindow->last)
-            twin_msecLast = timeWindow->last * 1000LL;
+        twin_msecFirst = timeWindow->msecFirst;
+        if (timeWindow->msecLast)
+            twin_msecLast = timeWindow->msecLast;
         else
             twin_msecLast = 0x7FFFFFFFFFFFFFFFLL;
     }
 
     recordHandle_t *recordHandle = calloc(1, sizeof(recordHandle_t));
     if (recordHandle == NULL) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         exit(255);
     }
 
@@ -421,7 +427,7 @@ __attribute__((noreturn)) static void *filterThread(void *arg) {
                     dataBlock->NumRecords = i;
                     break;
                 }
-                LogError("Corrupt data file. Inconsistent block size in %s line %d\n", __FILE__, __LINE__);
+                LogError("Corrupt data file. Inconsistent block size in %s line %d", __FILE__, __LINE__);
                 LogError("DataBlock: count: %u, size: %u. Found: %u, size: %u", dataBlock->NumRecords, dataBlock->size, i, sumSize);
                 sumSize = 0;
                 break;
@@ -466,6 +472,7 @@ __attribute__((noreturn)) static void *filterThread(void *arg) {
                 case ExporterInfoRecordType:
                 case ExporterStatRecordType:
                 case SamplerRecordType:
+                case SamplerLegacyRecordType:
                 case NbarRecordType:
                 case IfNameRecordType:
                 case VrfNameRecordType:
@@ -509,6 +516,10 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
 
     // check numWorkers depending on cores online
     uint32_t numWorkers = GetNumWorkers(0);
+    if (numWorkers > MAX_FILTER_THREADS) {
+        LogError("Limit number of filter threads to %d", MAX_FILTER_THREADS);
+        numWorkers = MAX_FILTER_THREADS;
+    }
     filterArgs_t filterArgs = {
         .engine = engine,
         .numWorkers = numWorkers,
@@ -519,7 +530,7 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
     };
     queue_producers(filterArgs.processQueue, numWorkers);
 
-    pthread_t tidFilter[32];
+    pthread_t tidFilter[MAX_FILTER_THREADS];
     for (int i = 0; i < numWorkers; i++) {
         int err = pthread_create(&(tidFilter[i]), NULL, filterThread, (void *)&filterArgs);
         if (err) {
@@ -535,7 +546,6 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
         nffile_w = OpenNewFile(wfile, NULL, CREATOR_NFDUMP, compress, NOT_ENCRYPTED);
         if (!nffile_w) {
             stat_record.firstseen = 0;
-            // XXX DisposeFile(nffile_r);
             return stat_record;
         }
         dataBlock_w = WriteBlock(nffile_w, NULL);
@@ -558,6 +568,7 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
         record_header_t *record_ptr = GetCursor(dataBlock);
 
         uint64_t recordCounter = dataHandle->recordCnt;
+        outputParams->ident = dataHandle->ident;
 
         // successfully read block
         total_bytes += dataBlock->size;
@@ -600,7 +611,7 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
                             dataBlock_w = AppendToBuffer(nffile_w, dataBlock_w, (void *)record_ptr, record_ptr->size);
                             break;
                         case PRINTRECORD:
-                            print_record(stdout, recordHandle, outputParams->doTag);
+                            print_record(stdout, recordHandle, outputParams);
                             break;
                     }
 
@@ -611,7 +622,10 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
                 case ExporterInfoRecordType: {
                     int ret = AddExporterInfo((exporter_info_record_t *)record_ptr);
                     if (ret != 0) {
-                        if (nffile_w) dataBlock_w = AppendToBuffer(nffile_w, dataBlock_w, (void *)record_ptr, record_ptr->size);
+                        if (nffile_w) {
+                            dbg_printf("Dump ExporterInfo Record to file\n");
+                            dataBlock_w = AppendToBuffer(nffile_w, dataBlock_w, (void *)record_ptr, record_ptr->size);
+                        }
                     } else {
                         LogError("Failed to add Exporter Record\n");
                     }
@@ -620,12 +634,27 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
                     AddExporterStat((exporter_stats_record_t *)record_ptr);
                     break;
                 case SamplerLegacyRecordType: {
-                    if (AddSamplerLegacyRecord((samplerV0_record_t *)record_ptr) == 0) LogError("Failed to add legacy Sampler Record\n");
+                    sampler_record_t *sampler_record = ConvertLegacyRecord((samplerV0_record_t *)record_ptr);
+                    if (sampler_record != NULL) {
+                        int ret = AddSamplerRecord(sampler_record);
+                        if (ret != 0) {
+                            if (nffile_w) {
+                                dbg_printf("Dump converted Sampler Record to file\n");
+                                dataBlock_w = AppendToBuffer(nffile_w, dataBlock_w, (void *)sampler_record, sampler_record->size);
+                            }
+                        } else {
+                            LogError("Failed to add converted Sampler Record\n");
+                        }
+                        free(sampler_record);
+                    }
                 } break;
                 case SamplerRecordType: {
                     int ret = AddSamplerRecord((sampler_record_t *)record_ptr);
                     if (ret != 0) {
-                        if (nffile_w) dataBlock_w = AppendToBuffer(nffile_w, dataBlock_w, (void *)record_ptr, record_ptr->size);
+                        if (nffile_w) {
+                            dbg_printf("Dump Sampler Record to file\n");
+                            dataBlock_w = AppendToBuffer(nffile_w, dataBlock_w, (void *)record_ptr, record_ptr->size);
+                        }
                     } else {
                         LogError("Failed to add Sampler Record\n");
                     }
@@ -664,6 +693,10 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
 
         // free resources
         FreeDataBlock(dataHandle->dataBlock);
+        if (dataHandle->ident) {
+            outputParams->ident = dataHandle->ident;
+            free(dataHandle);
+        }
     }  // while
 
     dbg_printf("processData() done\n");
@@ -672,6 +705,7 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
     if (nffile_w) {
         // flush current buffer to disc
         FlushBlock(nffile_w, dataBlock_w);
+        SetIdent(nffile_w, outputParams->ident);
 
         /* Copy stat info and close file */
         memcpy((void *)nffile_w->stat_record, (void *)&stat_record, sizeof(stat_record_t));
@@ -699,7 +733,6 @@ static stat_record_t process_data(void *engine, int processMode, char *wfile, Re
 }  // End of process_data
 
 int main(int argc, char **argv) {
-    struct stat stat_buff;
     stat_record_t sum_stat;
     outputParams_t *outputParams;
     RecordPrinter_t print_record;
@@ -707,7 +740,7 @@ int main(int argc, char **argv) {
     char *wfile, *ffile, *filter, *tstring, *stat_type;
     char *print_format;
     char *print_order, *query_file, *configFile, *nameserver, *aggr_fmt;
-    int ffd, element_stat, fdump;
+    int element_stat, fdump;
     int flow_stat, aggregate, aggregate_mask, bidir;
     int print_stat, gnuplot_stat, syntax_only, compress, worker;
     int GuessDir, ModifyCompress;
@@ -749,7 +782,7 @@ int main(int argc, char **argv) {
 
     outputParams = (outputParams_t *)calloc(1, sizeof(outputParams_t));
     if (!outputParams) {
-        LogError("calloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("calloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         exit(EXIT_FAILURE);
     }
     outputParams->topN = -1;
@@ -878,7 +911,6 @@ int main(int argc, char **argv) {
                 CheckArgLen(optarg, 64);
                 stat_type = optarg;
                 if (!SetStat(stat_type, &element_stat, &flow_stat)) {
-                    ListStatTypes();
                     exit(EXIT_FAILURE);
                 }
                 break;
@@ -890,7 +922,7 @@ int main(int argc, char **argv) {
                 outputParams->printPlain = 1;
                 break;
             case 'f':
-                CheckArgLen(optarg, MAXPATHLEN);
+                if (!CheckPath(optarg, S_IFREG)) exit(255);
                 ffile = optarg;
                 break;
             case 't':
@@ -925,7 +957,7 @@ int main(int argc, char **argv) {
                 ret = Parse_PrintOrder(print_order);
                 if (ret < 0) {
                     LogError("Unknown print order '%s'", print_order);
-                    ListPrintOrder();
+                    ListFlowPrintOrder();
                     exit(EXIT_FAILURE);
                 }
             } break;
@@ -1027,28 +1059,10 @@ int main(int argc, char **argv) {
     }
 
     if (!filter && ffile) {
-        if (stat(ffile, &stat_buff)) {
-            LogError("Can't stat filter file '%s': %s", ffile, strerror(errno));
-            exit(EXIT_FAILURE);
+        filter = ReadFilter(ffile);
+        if (filter == NULL) {
+            exit(255);
         }
-        filter = (char *)malloc(stat_buff.st_size + 1);
-        if (!filter) {
-            LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
-            exit(EXIT_FAILURE);
-        }
-        ffd = open(ffile, O_RDONLY);
-        if (ffd < 0) {
-            LogError("Can't open filter file '%s': %s", ffile, strerror(errno));
-            exit(EXIT_FAILURE);
-        }
-        ssize_t ret = read(ffd, (void *)filter, stat_buff.st_size);
-        if (ret < 0) {
-            LogError("Error reading filter file %s: %s", ffile, strerror(errno));
-            close(ffd);
-            exit(EXIT_FAILURE);
-        }
-        filter[stat_buff.st_size] = 0;
-        close(ffd);
 
         FilterFilename = ffile;
     }
@@ -1259,6 +1273,7 @@ int main(int argc, char **argv) {
         if (wfile) {
             nffile_t *nffile = OpenNewFile(wfile, NULL, CREATOR_NFDUMP, compress, NOT_ENCRYPTED);
             if (!nffile) exit(EXIT_FAILURE);
+            SetIdent(nffile, outputParams->ident);
             if (ExportFlowTable(nffile, aggregate, bidir, GuessDir)) {
                 CloseUpdateFile(nffile);
             } else {
@@ -1290,17 +1305,15 @@ int main(int argc, char **argv) {
             case MODE_NULL:
             case MODE_FMT:
                 PrintSummary(&sum_stat, outputParams);
-                if (t_last_flow == 0) {
+                if (t_lastMsec == 0) {
                     printf("Time window: <unknown>\n");
                 } else {
-                    t_first_flow /= 1000LL;
-                    t_last_flow /= 1000LL;
                     if (flist.timeWindow) {
-                        if (flist.timeWindow->first && (flist.timeWindow->first > t_first_flow)) t_first_flow = flist.timeWindow->first;
-                        if (flist.timeWindow->last && (flist.timeWindow->last < t_last_flow)) t_last_flow = flist.timeWindow->last;
+                        if (flist.timeWindow->msecFirst && (flist.timeWindow->msecFirst > t_firstMsec)) t_firstMsec = flist.timeWindow->msecFirst;
+                        if (flist.timeWindow->msecLast && (flist.timeWindow->msecLast < t_lastMsec)) t_lastMsec = flist.timeWindow->msecLast;
                     }
-                    double duration = (double)(t_last_flow - t_first_flow);
-                    printf("Time window: %s, Duration:%s\n", TimeString(t_first_flow, t_last_flow), DurationString(duration));
+                    uint64_t durationMsec = t_lastMsec - t_firstMsec;
+                    printf("Time window: %s, Duration:%s\n", TimeString(t_firstMsec, t_lastMsec), DurationString(durationMsec));
                 }
                 printf("Total records processed: %" PRIu64 ", passed: %" PRIu64 ", Blocks skipped: %u, Bytes read: %llu\n", totalRecords, totalPassed,
                        skippedBlocks, (unsigned long long)total_bytes);
