@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2024, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -76,11 +76,15 @@ static struct maskArray_s {
 static uint32_t maskIndex = 1;
 
 // For automatic output format generation in case of custom aggregation
+// prepends start and duration time
 #define AggrPrependFmt "%ts %td "
 #define AggrPrependCvs "%ts,%td"
+// appends counters
 #define AggrAppendFmt "%pkt %byt %bps %bpp %fl"
 #define AggrAppendCvs "%pkt,%byt,%bps,%bpp,%fl"
 
+typedef enum { AUTOFMT = 0, AUTOCSV, USERFMT } aggrPrintMode_t;
+
 static struct aggregationElement_s {
     char *aggrElement;        // name of aggregation parameter
     aggregate_param_t param;  // the parameter array
@@ -149,7 +153,8 @@ static struct aggregationElement_s {
                         {"srcgeo", {EXlocal, OFFgeoSrcIP, SizeGEOloc, 0}, 0, SRC_GEO, 0, 0, "%sc"},
                         {"dstgeo", {EXlocal, OFFgeoDstIP, SizeGEOloc, 0}, 0, DST_GEO, 0, 0, "%dc"},
                         {"ethertype", {EXlayer2ID, OFFetherType, SIZEetherType, 0}, 0, NOPREPROCESS, 0, 0, "%eth"},
-                        {"ttl", {EXipInfoID, OFFipTTL, SIZEipTTL, 0}, 0, NOPREPROCESS, 0, 0, "%ttl"},
+                        {"minttl", {EXipInfoID, OFFminTTL, SIZEminTTL, 0}, 0, NOPREPROCESS, 0, 0, "%minttl"},
+                        {"maxttl", {EXipInfoID, OFFmaxTTL, SIZEmaxTTL, 0}, 0, NOPREPROCESS, 0, 0, "%maxttl"},
                         {NULL, {0, 0, 0}, 0, NOPREPROCESS, 0, 0, NULL}};
 
 // FlowHash stat record, to aggregate flow counters in -A or -s stat/aggregate mode
@@ -694,7 +699,6 @@ static inline void PreProcess(void *inPtr, preprocess_t process, recordHandle_t
 
     switch (process) {
         case NOPREPROCESS:
-            return;
             break;
         case SRC_GEO: {
             char *geo = (char *)inPtr;
@@ -787,7 +791,7 @@ static inline int New_HashKey(void *keymem, recordHandle_t *recordHandle, int sw
                 case 16: {
                     ((uint64_t *)keymem)[0] = ((uint64_t *)inPtr)[0];
                     ((uint64_t *)keymem)[1] = ((uint64_t *)inPtr)[1];
-                    keymem += sizeof(uint64_t);
+                    keymem += (2 * sizeof(uint64_t));
                 } break;
                 default:
                     memcpy((void *)keymem, inPtr, param->length);
@@ -843,7 +847,7 @@ static inline int New_HashKey(void *keymem, recordHandle_t *recordHandle, int sw
             keyLen = sizeof(FlowKeyV6_t);
         } else {
             // catch all cases, actually not needed.
-            LogError("ipv4Flow: %d, ipv6Flow: %d, maxKeyLen: %u", ipv4Flow != NULL, ipv6Flow != NULL, maxKeyLen);
+            LogError("ipv4Flow: %d, ipv6Flow: %d, maxKeyLen: %zu", ipv4Flow != NULL, ipv6Flow != NULL, maxKeyLen);
             memset(keymem, 0, maxKeyLen);
         }
     }
@@ -966,6 +970,15 @@ void Dispose_FlowTable(void) {
     nfalloc_free();
 }  // End of Dispose_FlowTable
 
+void ListFlowPrintOrder(void) {
+    printf("Available flow print order:");
+    for (int i = 1; order_mode[i].string != NULL; i++) {
+        if (((i - 1) & 0x7) == 0) printf("\n");
+        printf(" %-9s", order_mode[i].string);
+    }
+    printf("\n See also nfdump(1)\n");
+}  // End of istFlowPrintOrder
+
 // Parse flow cache print order -O
 int Parse_PrintOrder(char *order) {
     dbg_printf("Enter %s\n", __func__);
@@ -1044,6 +1057,7 @@ int SetRecordStat(char *statType, char *optOrder) {
         }
         if (order_mode[i].string == NULL) {
             LogError("Unknown order option /%s", optOrder);
+            ListFlowPrintOrder();
             return 0;
         }
         FlowStat_order |= (1 << i);
@@ -1057,6 +1071,46 @@ int SetRecordStat(char *statType, char *optOrder) {
     return 1;
 }  // End of SetRecordStat
 
+// parse user supplied output format string
+// return NULL (invalid) if non aggregated elements are used in string
+static char *ParseAggrOutputFormat(char *print_format, char *arg) {
+    char *s = strdup(print_format);
+
+    // format already checked for fmt: or csv: - advance to token list
+    s += 4;
+
+    while (*s) {
+        if (*s == '%') {
+            char *token = s++;
+            while (*s && isalnum(*s)) s++;
+            char sep = *s;
+            *s = '\0';
+            // check, if this token is active
+            int active = 0;
+            int found = 0;
+            int index = 0;
+            while (aggregationTable[index].aggrElement) {
+                if (aggregationTable[index].fmt && strcmp(token, aggregationTable[index].fmt) == 0) {
+                    found = 1;
+                    active += aggregationTable[index].active;
+                }
+                index++;
+            }
+
+            if (found == 1 && active == 0) {
+                // token not active
+                LogError("Output token %s not allowed with current aggregation %s", token, arg);
+                return NULL;
+            }
+            *s = sep;
+        } else {
+            s++;
+        }
+    }
+    dbg_printf("Custom output format: %s\n", print_format);
+    return print_format;
+}  // End of ParseAggrOutputFormat
+
 char *ParseAggregateMask(char *print_format, char *arg) {
     dbg_printf("Enter %s\n", __func__);
     if (bidir_flows) {
@@ -1064,19 +1118,21 @@ char *ParseAggregateMask(char *print_format, char *arg) {
         return NULL;
     }
 
-    int modeCSV = 0;
+    aggrPrintMode_t aggrPrintMode = AUTOFMT;
     char *sep = " ";
     char *prepend = AggrPrependFmt;
     char *append = AggrAppendFmt;
     if (print_format) {
         if (strcmp(print_format, "csv") == 0) {
-            modeCSV = 1;
+            aggrPrintMode = AUTOCSV;
             sep = ",";
             prepend = AggrPrependCvs;
             append = AggrAppendCvs;
+        } else if (strncmp(print_format, "fmt:", 4) == 0 || strncmp(print_format, "csv:", 4) == 0) {
+            aggrPrintMode = USERFMT;
         } else {
-            printf("Can not use print format %s to aggregate flows\n", print_format);
-            exit(EXIT_FAILURE);
+            LogError("Invalid print format %s to aggregate flows", print_format);
+            return NULL;
         }
     }
 
@@ -1086,34 +1142,37 @@ char *ParseAggregateMask(char *print_format, char *arg) {
     maxKeyLen = 0;
     memset((void *)&aggregateInfo, 0, sizeof(aggregateInfo));
 
-    size_t fmt_len = 0;
+    size_t fmtLen = 0;
     for (int i = 0; aggregationTable[i].aggrElement != NULL; i++) {
-        if (modeCSV == 0 && HasGeoDB && aggregationTable[i].fmt) {
+        if (aggrPrintMode == AUTOFMT && HasGeoDB && aggregationTable[i].fmt) {
             if (strcmp(aggregationTable[i].fmt, "%sa") == 0) aggregationTable[i].fmt = "%gsa";
             if (strcmp(aggregationTable[i].fmt, "%da") == 0) aggregationTable[i].fmt = "%gda";
         }
-        if (aggregationTable[i].fmt) fmt_len += (strlen(aggregationTable[i].fmt) + 1);
+        if (aggregationTable[i].fmt) fmtLen += (strlen(aggregationTable[i].fmt) + 1);
     }
-    fmt_len++;  // max fmt string len incl. trailing '\0'
+    fmtLen++;  // max fmt string len incl. trailing '\0'
 
     // add format prepend and append length
-    fmt_len += strlen(prepend) + strlen(append) + 6;  // +6 for 'fmt:', 2 spaces
+    fmtLen += strlen(prepend) + strlen(append) + 6;  // +6 for 'fmt:', 2 spaces
 
-    char *aggr_fmt = (char *)malloc(fmt_len);
-    if (!aggr_fmt) {
+    // create auto format output mode while scanning aggregation
+    // will be discarded, if user supplies format
+    char *formatStr = (char *)malloc(fmtLen);
+    if (!formatStr) {
         LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return 0;
     }
-    aggr_fmt[0] = '\0';
-    if (modeCSV)
-        fmt_len -= snprintf(aggr_fmt, fmt_len, "csv:%s,", prepend);
+    formatStr[0] = '\0';
+    if (aggrPrintMode == AUTOCSV)
+        fmtLen -= snprintf(formatStr, fmtLen, "csv:%s,", prepend);
     else
-        fmt_len -= snprintf(aggr_fmt, fmt_len, "fmt:%s ", prepend);
+        fmtLen -= snprintf(formatStr, fmtLen, "fmt:%s ", prepend);
 
     uint32_t v4Mask = 0xffffffff;
     uint64_t v6Mask[2] = {0xffffffffffffffffLL, 0xffffffffffffffffLL};
     // separate tokens
-    char *p = strtok(arg, ",");
+    char *aggrStr = strdup(arg);
+    char *p = strtok(aggrStr, ",");
     while (p) {
         uint32_t has_mask = 0;
         // check for subnet bits
@@ -1128,6 +1187,7 @@ char *ParseAggregateMask(char *print_format, char *arg) {
                 // IPv4
                 if (subnet < 1 || subnet > 32) {
                     LogError("Subnet mask length '%d' out of range for IPv4", subnet);
+                    free(aggrStr);
                     return NULL;
                 }
                 v4Mask = 0xffffffff << (32 - subnet);
@@ -1137,6 +1197,7 @@ char *ParseAggregateMask(char *print_format, char *arg) {
                 // IPv6
                 if (subnet < 1 || subnet > 128) {
                     LogError("Subnet mask length '%d' out of range for IPv4", subnet);
+                    free(aggrStr);
                     return NULL;
                 }
 
@@ -1152,6 +1213,7 @@ char *ParseAggregateMask(char *print_format, char *arg) {
                 // rubbish
                 *q = '/';
                 LogError("Need src4/dst4 or src6/dst6 for IPv4 or IPv6 to aggregate with explicit netmask: '%s'", p);
+                free(aggrStr);
                 return NULL;
             }
         }
@@ -1161,32 +1223,36 @@ char *ParseAggregateMask(char *print_format, char *arg) {
 
         if (aggregationTable[index].aggrElement == NULL) {
             LogError("Unknown aggregation field '%s'", p);
+            free(aggrStr);
             return NULL;
         }
 
         if (aggregationTable[index].active) {
             LogError("Duplicate aggregation element: %s", p);
+            free(aggrStr);
             return NULL;
         }
 
         if (has_mask) {
             if (aggregationTable[index].allowMask == 0) {
                 LogError("Element %s does not take any netmask", p);
+                free(aggrStr);
                 return NULL;
             }
         }
 
         if (aggregationTable[index].fmt != NULL) {
-            strncat(aggr_fmt, aggregationTable[index].fmt, fmt_len);
-            fmt_len -= strlen(aggregationTable[index].fmt);
-            strncat(aggr_fmt, sep, fmt_len);
-            fmt_len -= 1;
+            strncat(formatStr, aggregationTable[index].fmt, fmtLen);
+            fmtLen -= strlen(aggregationTable[index].fmt);
+            strncat(formatStr, sep, fmtLen);
+            fmtLen -= 1;
         }
 
         do {
             // loop over alternate extensions v4/v6
             if (maskIndex >= MaxMaskArraySize) {
                 LogError("Too many netmasks");
+                free(aggrStr);
                 return NULL;
             }
             if (has_mask) {
@@ -1214,13 +1280,14 @@ char *ParseAggregateMask(char *print_format, char *arg) {
 
     if (elementCount == 0) {
         LogError("No aggregation specified!");
+        free(aggrStr);
         return NULL;
     }
     aggregateInfo[elementCount] = -1;
 
 #ifdef DEVEL
     printf("Aggregate key:  maxKeyLen: %zu bytes\n", maxKeyLen);
-    printf("Aggregate format string: '%s'\n", aggr_fmt);
+    printf("Aggregate format string: '%s'\n", formatStr);
 
     printf("Aggregate stack:\n");
     for (int i = 0; aggregateInfo[i] >= 0; i++) {
@@ -1232,18 +1299,31 @@ char *ParseAggregateMask(char *print_format, char *arg) {
             if (aggregationTable[index].netmaskID && aggregationTable[index].netmaskID != 0xFF) {
                 uint32_t maskIndex = aggregationTable[index].netmaskID;
                 printf("v4mask  : 0x%x\n", maskArray[maskIndex].v4Mask);
-                printf("v6mask  : 0x%llx 0x%llx\n", maskArray[maskIndex].v6Mask[0], maskArray[maskIndex].v6Mask[1]);
+                printf("v6mask  : 0x%" PRIx64 " 0x%" PRIx64 "\n", maskArray[maskIndex].v6Mask[0], maskArray[maskIndex].v6Mask[1]);
             }
         }
     }
 
 #endif
-    if (modeCSV == 0) {
-        strncat(aggr_fmt, sep, fmt_len);
-        fmt_len--;
+
+    if (aggrPrintMode == USERFMT) {
+        // check user supplied output format - discard auto format
+        free(formatStr);
+        char *formatStr = ParseAggrOutputFormat(print_format, arg);
+        free(aggrStr);
+        return formatStr;
     }
-    strncat(aggr_fmt, append, fmt_len);
-    return aggr_fmt;
+
+    // else for auto created output format, add 'append' format
+    if (aggrPrintMode == AUTOFMT) {
+        strncat(formatStr, sep, fmtLen);
+        fmtLen--;
+    }
+    strncat(formatStr, append, fmtLen);
+
+    dbg_printf("Auto output format: %s\n", formatStr);
+    free(aggrStr);
+    return formatStr;
 
 }  // End of ParseAggregateMask
 
@@ -1448,7 +1528,7 @@ void AddFlowCache(recordHandle_t *recordHandle) {
     uint64_t aggrFlows = 1;
     if (cntFlow) {
         outPackets = cntFlow->outPackets;
-        outBytes = cntFlow->outPackets;
+        outBytes = cntFlow->outBytes;
         aggrFlows = cntFlow->flows ? cntFlow->flows : 1;
     }
 
@@ -1546,7 +1626,7 @@ static SortElement_t *GetSortList(uint64_t *size) {
     if (hashSize) {  // hash table
         list = (SortElement_t *)calloc(hashSize, sizeof(SortElement_t));
         if (!list) {
-            LogError("calloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+            LogError("calloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             return NULL;
         }
 
@@ -1562,7 +1642,7 @@ static SortElement_t *GetSortList(uint64_t *size) {
         }
         list = (SortElement_t *)calloc(listSize, sizeof(SortElement_t));
         if (!list) {
-            LogError("calloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+            LogError("calloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
             return NULL;
         }
 
@@ -1610,9 +1690,15 @@ static inline void PrintSortList(SortElement_t *SortList, uint64_t maxindex, out
         genericFlow->tcpFlags = flowRecord->inFlags;
 
         EXcntFlow_t tmpCntFlow = {0};
-        if (cntFlow == NULL && (flowRecord->flows > 1 || flowRecord->outPackets)) {
-            recordHandle.extensionList[EXcntFlowID] = &tmpCntFlow;
-            cntFlow = &tmpCntFlow;
+        if (cntFlow == NULL) {
+            if (flowRecord->flows > 1 || flowRecord->outPackets) {
+                recordHandle.extensionList[EXcntFlowID] = &tmpCntFlow;
+                cntFlow = &tmpCntFlow;
+                cntFlow->outPackets = flowRecord->outPackets;
+                cntFlow->outBytes = flowRecord->outBytes;
+                cntFlow->flows = flowRecord->flows;
+            }
+        } else {
             cntFlow->outPackets = flowRecord->outPackets;
             cntFlow->outBytes = flowRecord->outBytes;
             cntFlow->flows = flowRecord->flows;
@@ -1624,14 +1710,66 @@ static inline void PrintSortList(SortElement_t *SortList, uint64_t maxindex, out
         }
 
         if (outputParams->postFilter) {
-            if (FilterRecord(outputParams->postFilter, &recordHandle)) print_record(stdout, &recordHandle, outputParams->doTag);
+            if (FilterRecord(outputParams->postFilter, &recordHandle)) print_record(stdout, &recordHandle, outputParams);
         } else {
-            print_record(stdout, &recordHandle, outputParams->doTag);
+            print_record(stdout, &recordHandle, outputParams);
         }
     }
 
 }  // End of PrintSortList
 
+// in case of custom aggregation, we need to rebuild the record
+// with only those elements aggregated
+static inline void RebuildRecord(void *buffPtr, recordHeaderV3_t *recordHeaderV3, uint64_t i) {
+    if (aggregateInfo[0] >= 0) {
+        // custom user aggregation - create new record
+        void *newExtensionList[MAXLISTSIZE] = {0};
+        AddV3Header(buffPtr, newV3Record);
+        PushExtension(newV3Record, EXgenericFlow, newGenericFlow);
+        newV3Record->flags = recordHeaderV3->flags;
+        newV3Record->nfversion = recordHeaderV3->nfversion;
+
+        // orig record
+        recordHandle_t recordHandle = {0};
+        MapRecordHandle(&recordHandle, recordHeaderV3, i);
+
+        // process record
+        // copy all custom aggregation elements
+        for (int j = 0; aggregateInfo[j] >= 0; j++) {
+            uint32_t tableIndex = aggregateInfo[j];
+            uint16_t extID = aggregationTable[tableIndex].param.extID;
+            if (recordHandle.extensionList[extID]) {
+                // extension exists in orig record
+                void *newRecord = NULL;
+                if (newExtensionList[extID] == NULL) {
+                    // add this extension to the output record as well
+                    PushExtensionID(newV3Record, extID, extension);
+                    newExtensionList[extID] = extension;
+                    newRecord = extension;
+                } else {
+                    newRecord = newExtensionList[extID];
+                }
+                pointer_addr_t offset = aggregationTable[tableIndex].param.offset;
+                pointer_addr_t length = aggregationTable[tableIndex].param.length;
+                memcpy(newRecord + offset, recordHandle.extensionList[extID] + offset, length);
+            }
+        }
+
+        // copy timestamps
+        EXgenericFlow_t *origGenericFlow = (EXgenericFlow_t *)recordHandle.extensionList[EXgenericFlowID];
+        if (origGenericFlow && newGenericFlow) {
+            newGenericFlow->msecFirst = origGenericFlow->msecFirst;
+            newGenericFlow->msecLast = origGenericFlow->msecLast;
+            newGenericFlow->msecReceived = origGenericFlow->msecReceived;
+            newGenericFlow->inBytes = origGenericFlow->inBytes;
+            newGenericFlow->inPackets = origGenericFlow->inPackets;
+        }
+    } else {
+        // copy orig record
+        memcpy(buffPtr, (void *)recordHeaderV3, recordHeaderV3->size);
+    }
+}  // End of RebuildRecord
+
 // export SortList - apply possible aggregation mask to zero out aggregated fields
 static inline void ExportSortList(SortElement_t *SortList, uint64_t maxindex, nffile_t *nffile, int GuessFlowDirection, int ascending) {
     dbg_printf("Enter %s\n", __func__);
@@ -1658,7 +1796,9 @@ static inline void ExportSortList(SortElement_t *SortList, uint64_t maxindex, nf
 
         // write record
         void *buffPtr = GetCurrentCursor(dataBlock);
-        memcpy(buffPtr, (void *)recordHeaderV3, recordHeaderV3->size);
+
+        // prepare record to export into new file
+        RebuildRecord(buffPtr, recordHeaderV3, i);
 
         // remap header to written memory
         recordHeaderV3 = (recordHeaderV3_t *)buffPtr;
