@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2024, Peter Haag
+ *  Copyright (c) 2009-2025, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *
@@ -77,8 +77,7 @@ typedef enum {
     IS_ASORG
 } elementType_t;
 
-typedef enum { DESCENDING = 0,
-               ASCENDING } direction_t;
+typedef enum { DESCENDING = 0, ASCENDING } direction_t;
 
 /*
  * pre-process functions:
@@ -205,18 +204,18 @@ static struct StatParameter_s {
     {"event", " Event", {EXnselCommonID, OFFfwEvent, SIZEfwEvent, 0}, IS_EVENT, NULL},
     {"xevent", " Event", {EXnselCommonID, OFFfwXevent, SIZEfwXevent, 0}, IS_NUMBER, NULL},
     {"nat", "NAT Event", {EXnatCommonID, OFFnatEvent, SIZEnatEvent, 0}, IS_EVENT, NULL},
-    {"xsrcip", "X-Src IP Addr", {EXnatXlateIPv4ID, OFFxlateSrc4Addr, SIZExlateSrc4Addr, AF_INET}, IS_IPADDR, NULL},
-    {"xsrcip", NULL, {EXnatXlateIPv6ID, OFFxlateSrc6Addr, SIZExlateSrc6Addr, AF_INET6}, IS_IPADDR, NULL},
-    {"xdstip", "X-Dst IP Addr", {EXnatXlateIPv4ID, OFFxlateDst4Addr, SIZExlateDst4Addr, AF_INET}, IS_IPADDR, NULL},
-    {"xdstip", NULL, {EXnatXlateIPv6ID, OFFxlateDst6Addr, SIZExlateDst6Addr, AF_INET6}, IS_IPADDR, NULL},
-    {"xip", "X-IP Addr", {EXnatXlateIPv4ID, OFFxlateSrc4Addr, SIZExlateSrc4Addr, AF_INET}, IS_IPADDR, NULL},
-    {"xip", NULL, {EXnatXlateIPv6ID, OFFxlateSrc6Addr, SIZExlateSrc6Addr, AF_INET6}, IS_IPADDR, NULL},
-    {"xip", NULL, {EXnatXlateIPv4ID, OFFxlateDst4Addr, SIZExlateDst4Addr, AF_INET}, IS_IPADDR, NULL},
-    {"xip", NULL, {EXnatXlateIPv6ID, OFFxlateDst6Addr, SIZExlateDst6Addr, AF_INET6}, IS_IPADDR, NULL},
-    {"xsrcport", "X-Src Port", {EXnatXlatePortID, OFFxlateSrcPort, SIZExlateSrcPort, 0}, IS_NUMBER, NULL},
-    {"xdstport", "X-Dst Port", {EXnatXlatePortID, OFFxlateDstPort, SIZExlateDstPort, 0}, IS_NUMBER, NULL},
-    {"xport", "X-Port", {EXnatXlatePortID, OFFxlateSrcPort, SIZExlateSrcPort, 0}, IS_NUMBER, NULL},
-    {"xport", NULL, {EXnatXlatePortID, OFFxlateDstPort, SIZExlateDstPort, 0}, IS_NUMBER, NULL},
+    {"natsrcip", "X-Src IP Addr", {EXnatXlateIPv4ID, OFFxlateSrc4Addr, SIZExlateSrc4Addr, AF_INET}, IS_IPADDR, NULL},
+    {"natsrcip", NULL, {EXnatXlateIPv6ID, OFFxlateSrc6Addr, SIZExlateSrc6Addr, AF_INET6}, IS_IPADDR, NULL},
+    {"natdstip", "X-Dst IP Addr", {EXnatXlateIPv4ID, OFFxlateDst4Addr, SIZExlateDst4Addr, AF_INET}, IS_IPADDR, NULL},
+    {"natdstip", NULL, {EXnatXlateIPv6ID, OFFxlateDst6Addr, SIZExlateDst6Addr, AF_INET6}, IS_IPADDR, NULL},
+    {"natip", "X-IP Addr", {EXnatXlateIPv4ID, OFFxlateSrc4Addr, SIZExlateSrc4Addr, AF_INET}, IS_IPADDR, NULL},
+    {"natip", NULL, {EXnatXlateIPv6ID, OFFxlateSrc6Addr, SIZExlateSrc6Addr, AF_INET6}, IS_IPADDR, NULL},
+    {"natip", NULL, {EXnatXlateIPv4ID, OFFxlateDst4Addr, SIZExlateDst4Addr, AF_INET}, IS_IPADDR, NULL},
+    {"natip", NULL, {EXnatXlateIPv6ID, OFFxlateDst6Addr, SIZExlateDst6Addr, AF_INET6}, IS_IPADDR, NULL},
+    {"natsrcport", "X-Src Port", {EXnatXlatePortID, OFFxlateSrcPort, SIZExlateSrcPort, 0}, IS_NUMBER, NULL},
+    {"natdstport", "X-Dst Port", {EXnatXlatePortID, OFFxlateDstPort, SIZExlateDstPort, 0}, IS_NUMBER, NULL},
+    {"natport", "X-Port", {EXnatXlatePortID, OFFxlateSrcPort, SIZExlateSrcPort, 0}, IS_NUMBER, NULL},
+    {"natport", NULL, {EXnatXlatePortID, OFFxlateDstPort, SIZExlateDstPort, 0}, IS_NUMBER, NULL},
     {"iacl", "Ingress ACL", {EXnselAclID, OFFingressAcl, SIZEingressAcl, 0}, IS_HEX, NULL},
     {"eacl", "Egress ACL", {EXnselAclID, OFFegressAcl, SIZEegressAcl, 0}, IS_HEX, NULL},
     // {"iace", "Ingress ACL", {EXnselAclID, OFFingressAcl, SIZEingressAcl, 0}, IS_HEX, NULL},
@@ -225,7 +224,8 @@ static struct StatParameter_s {
     // {"exace", "Egress ACL", {EXnselAclID, OFFegressAcl, SIZEegressAcl, 0}, IS_HEX, NULL},
     {"ivrf", "I-vrf ID", {EXvrfID, OFFingressVrf, SIZEingressVrf, 0}, IS_NUMBER, NULL},
     {"evrf", "E-vrf ID", {EXvrfID, OFFegressVrf, SIZEegressVrf, 0}, IS_NUMBER, NULL},
-    {"ttl", "TTL", {EXipInfoID, OFFipTTL, SIZEipTTL, 0}, IS_NUMBER, NULL},
+    {"minttl", "minTTL", {EXipInfoID, OFFminTTL, SIZEminTTL, 0}, IS_NUMBER, NULL},
+    {"maxttl", "maxTTL", {EXipInfoID, OFFmaxTTL, SIZEmaxTTL, 0}, IS_NUMBER, NULL},
 
     // header info != NULL
     {NULL, "DONE", {0, 0, 0, 0}, 0, NULL}};
@@ -257,9 +257,7 @@ typedef struct StatRecord {
  * orderby functions:
  * retrieve or calculate value, records want to be ordered by.
  */
-typedef enum flowDir { IN = 0,
-                       OUT,
-                       INOUT } flowDir_t;
+typedef enum flowDir { IN = 0, OUT, INOUT } flowDir_t;
 typedef uint64_t (*order_proc_element_t)(StatRecord_t *);
 
 static uint64_t order_bytes_in(StatRecord_t *record);
@@ -421,6 +419,10 @@ static StatRecord_t *elementHash_add(ElementHash_t *elementHash, hashkey_t *key,
 }
 
 /* function prototypes */
+static void ListStatPrintOrder(void);
+
+static void ListElementStatTypes(void);
+
 static int ParseListOrder(char *orderBy, struct StatRequest_s *request);
 
 static void PrintStatLine(stat_record_t *stat, outputParams_t *outputParams, SortElement_t *element, int type, int order_proto, int inout);
@@ -585,6 +587,7 @@ static int ParseListOrder(char *orderBy, struct StatRequest_s *request) {
                     direction = DESCENDING;
                     break;
                 default:
+                    LogError("Invalid direction %c. Use a for ascending or d for descending", *r);
                     return -1;
             }
         } else {
@@ -598,7 +601,6 @@ static int ParseListOrder(char *orderBy, struct StatRequest_s *request) {
         }
         if (orderByTable[i].string == NULL) {
             LogError("Unknown order option /%s", orderBy);
-            ListPrintOrder();
             return 0;
         }
         request->orderBy |= (1 << i);
@@ -637,6 +639,7 @@ int SetElementStat(char *elementStat, char *orderBy) {
             request->order_proto = 1;
         } else {
             LogError("Unknown statistic option :%s in %s", optProto, elementStat);
+            LogError("Available option :p for ordered by protocol");
             return 0;
         }
     }
@@ -654,12 +657,13 @@ int SetElementStat(char *elementStat, char *orderBy) {
 
     if (StatParameters[i].statname == NULL) {
         LogError("Unknown statistic: %s", elementStat);
-        ListStatTypes();
+        ListElementStatTypes();
         return 0;
     }
 
     // check if one or more orders are given
     if (ParseListOrder(orderBy, request) == 0) {
+        ListStatPrintOrder();
         return 0;
     }
 
@@ -914,10 +918,10 @@ static void PrintStatLine(stat_record_t *stat, outputParams_t *outputParams, Sor
         case IS_NULL:
             break;
         case IS_NUMBER:
-            snprintf(valstr, 64, "%llu", (unsigned long long)hashKey->v1);
+            snprintf(valstr, 64, "%" PRIu64, hashKey->v1);
             break;
         case IS_HEXNUMBER:
-            snprintf(valstr, 64, "0x%llx", (unsigned long long)hashKey->v1);
+            snprintf(valstr, 64, "0x%" PRIx64, hashKey->v1);
             break;
         case IS_IPADDR:
             tag_string[0] = outputParams->doTag ? TAG_CHAR : '\0';
@@ -954,7 +958,7 @@ static void PrintStatLine(stat_record_t *stat, outputParams_t *outputParams, Sor
             snprintf(valstr, 64, "%.2x:%.2x:%.2x:%.2x:%.2x:%.2x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);
         } break;
         case IS_MPLS_LBL: {
-            snprintf(valstr, 64, "%llu", (unsigned long long)hashKey->v1);
+            snprintf(valstr, 64, "%" PRIu64, (uint64_t)hashKey->v1);
             snprintf(valstr, 64, "%8llu-%1llu-%1llu", (unsigned long long)hashKey->v1 >> 4, ((unsigned long long)hashKey->v1 & 0xF) >> 1,
                      (unsigned long long)hashKey->v1 & 1);
         } break;
@@ -1074,7 +1078,7 @@ static void PrintStatLine(stat_record_t *stat, outputParams_t *outputParams, Sor
     time_t first = statRecord->msecFirst / 1000LL;
     struct tm *tbuff = localtime(&first);
     if (!tbuff) {
-        LogError("localtime() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("localtime() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return;
     }
     char datestr[64];
@@ -1117,10 +1121,10 @@ static void PrintJsonStatLine(char *statName, stat_record_t *stat, outputParams_
         case IS_NULL:
             break;
         case IS_NUMBER:
-            snprintf(valstr, 64, "%llu", (unsigned long long)hashKey->v1);
+            snprintf(valstr, 64, "%" PRIu64, (uint64_t)hashKey->v1);
             break;
         case IS_HEXNUMBER:
-            snprintf(valstr, 64, "0x%llx", (unsigned long long)hashKey->v1);
+            snprintf(valstr, 64, "0x%" PRIx64, hashKey->v1);
             break;
         case IS_IPADDR:
             if (hashKey->v0 == 0) {  // IPv4
@@ -1146,7 +1150,7 @@ static void PrintJsonStatLine(char *statName, stat_record_t *stat, outputParams_
             snprintf(valstr, 64, "%.2x:%.2x:%.2x:%.2x:%.2x:%.2x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);
         } break;
         case IS_MPLS_LBL: {
-            snprintf(valstr, 64, "%llu", (unsigned long long)hashKey->v1);
+            snprintf(valstr, 64, "%" PRIu64, (uint64_t)hashKey->v1);
             snprintf(valstr, 64, "%8llu-%1llu-%1llu", (unsigned long long)hashKey->v1 >> 4, ((unsigned long long)hashKey->v1 & 0xF) >> 1,
                      (unsigned long long)hashKey->v1 & 1);
         } break;
@@ -1269,7 +1273,7 @@ static void PrintCvsStatLine(stat_record_t *stat, int printPlain, SortElement_t
         case IS_NULL:
             break;
         case IS_NUMBER:
-            snprintf(valstr, 40, "%llu", (unsigned long long)hashKey->v1);
+            snprintf(valstr, 40, "%" PRIu64, (uint64_t)hashKey->v1);
             break;
         case IS_IPADDR:
             if (hashKey->v0 != 0) {  // IPv6
@@ -1293,7 +1297,7 @@ static void PrintCvsStatLine(stat_record_t *stat, int printPlain, SortElement_t
             snprintf(valstr, 40, "%.2x:%.2x:%.2x:%.2x:%.2x:%.2x", mac[5], mac[4], mac[3], mac[2], mac[1], mac[0]);
         } break;
         case IS_MPLS_LBL: {
-            snprintf(valstr, 40, "%llu", (unsigned long long)hashKey->v1);
+            snprintf(valstr, 40, "%" PRIu64, (uint64_t)hashKey->v1);
             snprintf(valstr, 40, "%8llu-%1llu-%1llu", (unsigned long long)hashKey->v1 >> 4, ((unsigned long long)hashKey->v1 & 0xF) >> 1,
                      (unsigned long long)hashKey->v1 & 1);
         } break;
@@ -1357,10 +1361,9 @@ static void PrintCvsStatLine(stat_record_t *stat, int printPlain, SortElement_t
     char datestr2[64];
     strftime(datestr2, 63, "%Y-%m-%d %H:%M:%S", tbuff);
 
-    printf("%s,%s,%.3f,%s,%s,%llu,%.1f,%llu,%.1f,%llu,%.1f,%llu,%llu,%u\n", datestr1, datestr2, duration,
-           order_proto ? ProtoString(hashKey->proto, printPlain) : "any", valstr, (long long unsigned)count_flows, flows_percent,
-           (long long unsigned)count_packets, packets_percent, (long long unsigned)count_bytes, bytes_percent, (long long unsigned)pps,
-           (long long unsigned)bps, bpp);
+    printf("%s,%s,%.3f,%s,%s,%" PRIu64 ",%.1f,%" PRIu64 ",%.1f,%" PRIu64 ",%.1f,%" PRIu64 ",%" PRIu64 ",%u\n", datestr1, datestr2, duration,
+           order_proto ? ProtoString(hashKey->proto, printPlain) : "any", valstr, count_flows, flows_percent, count_packets, packets_percent,
+           count_bytes, bytes_percent, pps, bps, bpp);
 
 }  // End of PrintCvsStatLine
 
@@ -1471,7 +1474,7 @@ static SortElement_t *StatTopN(int topN, uint32_t *count, int hash_num, int orde
     SortElement_t *topN_list = (SortElement_t *)calloc(numCells, sizeof(SortElement_t));
 
     if (!topN_list) {
-        LogError("calloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("calloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         return NULL;
     }
 
@@ -1484,7 +1487,7 @@ static SortElement_t *StatTopN(int topN, uint32_t *count, int hash_num, int orde
             record->hashkey = &(elemenHash->keys[i].key);
             c++;
 
-            dbg_printf("Get next hashCell. count: %llu\n", topN_list[c].count);
+            dbg_printf("Get next hashCell. count: %" PRIu64 "\n", topN_list[c].count);
         }
     }
     assert(c == numCells);
@@ -1497,16 +1500,17 @@ static SortElement_t *StatTopN(int topN, uint32_t *count, int hash_num, int orde
 
 }  // End of StatTopN
 
-void ListPrintOrder(void) {
-    printf("Available print order:");
+static void ListStatPrintOrder(void) {
+    printf("Available stat print order:");
     for (int i = 1; orderByTable[i].string != NULL; i++) {
         if (((i - 1) & 0x7) == 0) printf("\n");
         printf(" %-9s", orderByTable[i].string);
     }
-    printf("\n See also nfdump(1)\n");
-}  // End of ListPrintOrder
+    printf("\nOptionally add direction - :a for ascending or :d for descending values\n");
+    printf(" See also nfdump(1)\n");
+}  // End of ListStatPrintOrder
 
-void ListStatTypes(void) {
+static void ListElementStatTypes(void) {
     int cnt = 0;
     printf("Available element statistics:");
     for (int i = 0; StatParameters[i].statname != NULL; i++) {
@@ -1520,4 +1524,4 @@ void ListStatTypes(void) {
         };
     }
     printf("\n See also nfdump(1)\n");
-}  // End of ListStatTypes
+}  // End of ListElementStatTypes
