@@ -28,15 +28,15 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <arpa/inet.h>
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <inttypes.h>
 #include <libgen.h>
+
+#include "config.h"
+
 #ifdef FIX_INCLUDE
 #include <sys/types.h>
 #endif
@@ -85,6 +85,10 @@
 #include "util.h"
 #include "version.h"
 
+#ifdef HAVEZLIB
+#include "pcap_gzip.h"
+#endif
+
 #define TIME_WINDOW 300
 #define PROMISC 1
 #define TIMEOUT 500
@@ -172,8 +176,21 @@ static int setup_pcap_file(packetParam_t *param, char *pcap_file, char *filter,
     errbuf[0] = '\0';
     handle = pcap_open_offline(pcap_file, errbuf);
     if (handle == NULL) {
+#ifdef HAVEZLIB
+        FILE *fzip = zlib_stream(pcap_file);
+        if (fzip == NULL) {
+            LogError("Not a valid gzip format in %s", pcap_file);
+            return -1;
+        }
+        handle = pcap_fopen_offline(fzip, errbuf);
+        if (handle == NULL) {
+            LogError("pcap_fopen_offline() failed: %s", errbuf);
+            return -1;
+        }
+#else
         LogError("pcap_open_offline() failed: %s", errbuf);
         return -1;
+#endif
     }
 
     if (filter) {
@@ -651,8 +668,10 @@ int main(int argc, char *argv[]) {
     }
 
     if (pidfile) {
-        if (check_pid(pidfile) != 0 || write_pid(pidfile) == 0) pcap_close(packetParam.pcap_dev);
-        exit(EXIT_FAILURE);
+        if (check_pid(pidfile) != 0 || write_pid(pidfile) == 0) {
+            pcap_close(packetParam.pcap_dev);
+            exit(EXIT_FAILURE);
+        }
     }
 
     if (metricsocket && !OpenMetric(metricsocket, metricInterval)) {
@@ -764,7 +783,7 @@ int main(int argc, char *argv[]) {
 
     CloseMetric();
 
-    LogInfo("Total: Processed: %u, skipped: %u, short caplen: %u, unknown: %u, duplicates: %llu\n", packetParam.proc_stat.packets,
+    LogInfo("Total: Processed: %u, skipped: %u, short caplen: %u, unknown: %u, duplicates: %" PRIu64 "\n", packetParam.proc_stat.packets,
             packetParam.proc_stat.skipped, packetParam.proc_stat.short_snap, packetParam.proc_stat.unknown, packetParam.proc_stat.duplicates);
 
     if (pidfile) remove_pid(pidfile);
