@@ -33,6 +33,7 @@
 
 #include <pcap.h>
 #include <pthread.h>
+#include <time.h>
 
 #include "flowtree.h"
 #include "queue.h"
@@ -65,6 +66,7 @@ typedef struct proc_stat_s {
     uint32_t skipped;
     uint32_t unknown;
     uint32_t short_snap;
+    uint64_t duplicates;
 } proc_stat_t;
 
 #ifdef USE_TPACKETV3
@@ -94,8 +96,9 @@ typedef struct packetParam_s {
 
     NodeList_t *NodeList;
     pcap_t *pcap_dev;
-    int t_win;
+    time_t t_win;
     int *done;
+    int doDedup;
 
     uint32_t snaplen;
     uint32_t linktype;
