@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2024, Peter Haag
+ *  Copyright (c) 2024-2025, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -64,11 +64,17 @@ typedef char *(*string_function_t)(char *, recordHandle_t *);
 
 #include "itoa.c"
 
-#define AddString(s)               \
-    do {                           \
-        size_t len = strlen(s);    \
-        memcpy(streamPtr, s, len); \
-        streamPtr += len;          \
+// substitut ',' by ';' to keep csv clean
+#define AddString(s)                    \
+    do {                                \
+        size_t len = strlen((s));       \
+        const char *str = (s);          \
+        for (int i = 0; i < len; i++) { \
+            if (str[i] == ',')          \
+                *streamPtr++ = ';';     \
+            else                        \
+                *streamPtr++ = str[i];  \
+        }                               \
     } while (0)
 
 #define AddChar(c) *streamPtr++ = (c);
@@ -107,6 +113,8 @@ static double duration = 0;
 #define STRINGSIZE 10240
 static char header_string[STRINGSIZE] = {'\0'};
 
+static char *ident = NULL;
+
 /* prototypes */
 static char *ICMP_Port_decode(EXgenericFlow_t *genericFlow);
 
@@ -118,6 +126,8 @@ static void InitFormatParser(void);
 
 static void AddToken(int index);
 
+static char *String_Ident(char *streamPtr, recordHandle_t *recordHandle);
+
 static char *String_Version(char *streamPtr, recordHandle_t *recordHandle);
 
 static char *String_FlowCount(char *streamPtr, recordHandle_t *recordHandle);
@@ -216,7 +226,9 @@ static char *String_BiFlowDir(char *streamPtr, recordHandle_t *recordHandle);
 
 static char *String_FlowEndReason(char *streamPtr, recordHandle_t *recordHandle);
 
-static char *String_ipTTL(char *streamPtr, recordHandle_t *recordHandle);
+static char *String_ipminTTL(char *streamPtr, recordHandle_t *recordHandle);
+
+static char *String_ipmaxTTL(char *streamPtr, recordHandle_t *recordHandle);
 
 static char *String_ipFrag(char *streamPtr, recordHandle_t *recordHandle);
 
@@ -369,6 +381,7 @@ static struct format_entry_s {
     // csv format table
     {"%nfv", 0, "version", String_Version},      // netflow version
     {"%cnt", 0, "count", String_FlowCount},      // flow count
+    {"%idt", 0, "Ident", String_Ident},          // Ident string
     {"%eng", 0, "engine", String_Engine},        // Engine Type/ID
     {"%exp", 0, "exporterID", String_ExpSysID},  // Exporter SysID
 
@@ -562,8 +575,9 @@ static struct format_entry_s {
     {"%lbl", 0, "label", String_Label},                // Flow Label
 
     // EXipInfo
-    {"%ttl", 0, "TTL", String_ipTTL},     // Flow ip ttl
-    {"%frag", 0, "Frag", String_ipFrag},  // IP fragment flags
+    {"%minttl", 0, "minTTL", String_ipminTTL},  // Flow ip min ttl
+    {"%maxttl", 0, "maxTTL", String_ipmaxTTL},  // Flow ip max ttl
+    {"%frag", 0, "Frag", String_ipFrag},        // IP fragment flags
 
     {NULL, 0, NULL, NULL}};
 
@@ -582,7 +596,7 @@ static void ListOutputFormats(void) {
 
 }  // End of ListOutputFormats
 
-void csv_record(FILE *stream, recordHandle_t *recordHandle, int tag) {
+void csv_record(FILE *stream, recordHandle_t *recordHandle, outputParams_t *outputParam) {
     EXgenericFlow_t *genericFlow = (EXgenericFlow_t *)recordHandle->extensionList[EXgenericFlowID];
     EXtunIPv4_t *tunIPv4 = (EXtunIPv4_t *)recordHandle->extensionList[EXtunIPv4ID];
     EXtunIPv6_t *tunIPv6 = (EXtunIPv6_t *)recordHandle->extensionList[EXtunIPv6ID];
@@ -615,10 +629,11 @@ void csv_record(FILE *stream, recordHandle_t *recordHandle, int tag) {
             tunIPv6Flow->dstAddr[1] = tunIPv6->tunDstAddr[1];
             tunRecordHandle.extensionList[EXipv6FlowID] = tunIPv6Flow;
         }
-        csv_record(stream, &tunRecordHandle, tag);
+        csv_record(stream, &tunRecordHandle, outputParam);
         free(p);
     }
 
+    ident = outputParam->ident;
     streamBuff[0] = '\0';
     char *streamPtr = streamBuff;
     duration = 0;
@@ -626,7 +641,7 @@ void csv_record(FILE *stream, recordHandle_t *recordHandle, int tag) {
         if (genericFlow->msecLast >= genericFlow->msecFirst) {
             duration = (genericFlow->msecLast - genericFlow->msecFirst) / 1000.0;
         } else {
-            LogError("Record: %u Time error - last < first", recordHandle->flowCount);
+            LogError("Record: %" PRIu64 " Time error - last < first", recordHandle->flowCount);
             duration = 0;
         }
     }
@@ -800,6 +815,12 @@ static char *ICMP_Port_decode(EXgenericFlow_t *genericFlow) {
 }  // End of ICMP_Port_decode
 
 /* functions, which create the individual strings for the output line */
+static char *String_Ident(char *streamPtr, recordHandle_t *recordHandle) {
+    char *s = ident != NULL ? ident : "<no ident>";
+    AddString(s);
+    return streamPtr;
+}  // End of String_Ident
+
 static char *String_Version(char *streamPtr, recordHandle_t *recordHandle) {
     recordHeaderV3_t *recordHeaderV3 = recordHandle->recordHeaderV3;
 
@@ -1158,8 +1179,7 @@ static char *String_tlsVersion(char *streamPtr, recordHandle_t *recordHandle) {
     0x0302 = TLS 1.1 = “11”
     0x0301 = TLS 1.0 = “10”
     0x0300 = SSL 3.0 = “s3”
-    0x0200 = SSL 2.0 = “s2”
-    0x0100 = SSL 1.0 = “s1”
+    0x0002 = SSL 2.0 = “s2”
     */
 
     // ssl is defined
@@ -1731,14 +1751,23 @@ static char *String_FlowEndReason(char *streamPtr, recordHandle_t *recordHandle)
     return streamPtr;
 }  // End of String_FlowEndReason
 
-static char *String_ipTTL(char *streamPtr, recordHandle_t *recordHandle) {
+static char *String_ipminTTL(char *streamPtr, recordHandle_t *recordHandle) {
     EXipInfo_t *ipInfo = (EXipInfo_t *)recordHandle->extensionList[EXipInfoID];
-    uint8_t ttl = ipInfo ? ipInfo->ttl : 0;
+    uint8_t minTTL = ipInfo ? ipInfo->minTTL : 0;
 
-    AddU32(ttl);
+    AddU32(minTTL);
 
     return streamPtr;
-}  // End of String_ipTTL
+}  // End of String_ipminTTL
+
+static char *String_ipmaxTTL(char *streamPtr, recordHandle_t *recordHandle) {
+    EXipInfo_t *ipInfo = (EXipInfo_t *)recordHandle->extensionList[EXipInfoID];
+    uint8_t maxTTL = ipInfo ? ipInfo->maxTTL : 0;
+
+    AddU32(maxTTL);
+
+    return streamPtr;
+}  // End of String_ipminTTL
 
 static char *String_ipFrag(char *streamPtr, recordHandle_t *recordHandle) {
     EXipInfo_t *ipInfo = (EXipInfo_t *)recordHandle->extensionList[EXipInfoID];
