@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2019-2024, Peter Haag
+ *  Copyright (c) 2019-2025, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -273,18 +273,23 @@ static char *stringEXflowMisc(char *streamPtr, recordHandle_t *recordHandle, voi
 static char *stringEXipInfo(char *streamPtr, void *extensionRecord) {
     EXipInfo_t *ipInfo = (EXipInfo_t *)extensionRecord;
 
-    char flags[4] = "--\0";
-    if (ipInfo->fragmentFlags & flagDF) {
-        flags[0] = 'D';
-        flags[1] = 'F';
-    }
-    if (ipInfo->fragmentFlags & flagMF) {
-        flags[2] = 'M';
-        flags[3] = 'F';
-    }
+    if (ipInfo->fragmentFlags) {
+        char flags[4] = "--\0";
+        if (ipInfo->fragmentFlags & flagDF) {
+            flags[0] = 'D';
+            flags[1] = 'F';
+        }
+        if (ipInfo->fragmentFlags & flagMF) {
+            flags[2] = 'M';
+            flags[3] = 'F';
+        }
 
-    AddElementString("ip_fragment", flags);
-    AddElementU32("ip_ttl", (uint32_t)ipInfo->ttl);
+        AddElementString("ip_fragment", flags);
+    }
+    if (ipInfo->minTTL || ipInfo->maxTTL) {
+        AddElementU32("ip_minttl", (uint32_t)ipInfo->minTTL);
+        AddElementU32("ip_maxttl", (uint32_t)ipInfo->maxTTL);
+    }
 
     return streamPtr;
 }  // End of stringEXipInfo
@@ -418,7 +423,7 @@ static char *stringEXmplsLabel(char *streamPtr, void *extensionRecord) {
     EXmplsLabel_t *mplsLabel = (EXmplsLabel_t *)extensionRecord;
     for (int i = 0; i < 10; i++) {
         ptrdiff_t lenStream = STREAMLEN(streamPtr);
-        int len = snprintf(streamPtr, lenStream, "\"mpls_%u\":\"%u-%u-%u\",", i + 1, mplsLabel->mplsLabel[i] >> 4,
+        int len = snprintf(streamPtr, (size_t)lenStream, "\"mpls_%u\":\"%u-%u-%u\",", i + 1, mplsLabel->mplsLabel[i] >> 4,
                            (mplsLabel->mplsLabel[i] & 0xF) >> 1, mplsLabel->mplsLabel[i] & 1);
         streamPtr += len;
     }
@@ -786,7 +791,7 @@ void ndjson_epilog(outputParams_t *outputParam) {
 
 enum { FORMAT_NDJSON = 0, FORMAT_JSON };
 
-void flow_record_to_ndjson(FILE *stream, recordHandle_t *recordHandle, int tag) {
+void flow_record_to_ndjson(FILE *stream, recordHandle_t *recordHandle, outputParams_t *outputParam) {
     // ws is whitespace after object opening and before object closing {WS  WS}
     // indent is printed before each record for clarity if needed
     // fs is Field Separator
@@ -802,6 +807,8 @@ void flow_record_to_ndjson(FILE *stream, recordHandle_t *recordHandle, int tag)
     char *typeString = TestFlag(recordHeaderV3->flags, V3_FLAG_EVENT) ? "EVENT" : "FLOW";
     AddElementU32("cnt", ++recordCount);
     AddElementString("type", typeString);
+    if (outputParam->ident != NULL) AddElementString("ident", outputParam->ident);
+
     AddElementU32("export_sysid", recordHeaderV3->exporterID);
 
     int processed = 0;
