@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2019-2024, Peter Haag
+ *  Copyright (c) 2019-2025, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -306,13 +306,17 @@ static void stringsEXflowMisc(FILE *stream, recordHandle_t *recordHandle, void *
 static void stringEXipInfo(FILE *stream, void *extensionRecord) {
     EXipInfo_t *ipInfo = (EXipInfo_t *)extensionRecord;
 
-    char *DF = ipInfo->fragmentFlags & flagDF ? "DF" : "  ";
-    char *MF = ipInfo->fragmentFlags & flagMF ? "MF" : "  ";
-    fprintf(stream,
-            "  ip fragment  =               0x%.2x %s %s\n"
-            "  ip TTL       =              %5u\n",
-            ipInfo->fragmentFlags, DF, MF, ipInfo->ttl);
-
+    if (ipInfo->fragmentFlags) {
+        char *DF = ipInfo->fragmentFlags & flagDF ? "DF" : "  ";
+        char *MF = ipInfo->fragmentFlags & flagMF ? "MF" : "  ";
+        fprintf(stream, "  ip fragment  =               0x%.2x %s %s\n", ipInfo->fragmentFlags, DF, MF);
+    }
+    if (ipInfo->minTTL || ipInfo->maxTTL) {
+        fprintf(stream,
+                "  ip minTTL    =              %5u\n"
+                "  ip minTTL    =              %5u\n",
+                ipInfo->minTTL, ipInfo->maxTTL);
+    }
 }  // End of stringEXcntFlow
 
 static void stringsEXcntFlow(FILE *stream, void *extensionRecord) {
@@ -822,7 +826,7 @@ static void stringsEXinmon(FILE *stream, void *extensionRecord) {
 
 static void stringsEXflowId(FILE *stream, void *extensionRecord) {
     EXflowId_t *flowId = (EXflowId_t *)extensionRecord;
-    fprintf(stream, "  flow ID      = 0x%13" PRIx64 "\n", flowId->flowId);
+    fprintf(stream, "  flow ID      = %#18" PRIx64 "\n", flowId->flowId);
 }  // End of stringsEXflowId
 
 static void stringsEXnokiaNat(FILE *stream, void *extensionRecord) {
@@ -847,7 +851,7 @@ void raw_epilog(outputParams_t *outputParam) {
     // empty epilog
 }  // End of raw_epilog
 
-void raw_record(FILE *stream, recordHandle_t *recordHandle, int tag) {
+void raw_record(FILE *stream, recordHandle_t *recordHandle, outputParams_t *outputParam) {
     recordHeaderV3_t *recordHeaderV3 = recordHandle->recordHeaderV3;
 
     char elementString[MAXEXTENSIONS * 5];
@@ -886,6 +890,10 @@ void raw_record(FILE *stream, recordHandle_t *recordHandle, int tag) {
             "  RecordCount  =              %5" PRIu64 "\n",
             recordHandle->flowCount);
 
+    if (outputParam->ident) {
+        fprintf(stream, "  Ident        =       %12s\n", outputParam->ident);
+    }
+
     fprintf(stream,
             "  Flags        =               0x%.2x %s%s%s, %s\n"
             "  Elements     =              %5u: %s\n"
