@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2024, Peter Haag
+ *  Copyright (c) 2024-2025, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -241,6 +241,26 @@ static void runTest(void) {
     CheckFilter("bpp 2", recordHandle, 1);
     CheckFilter("bpp > 2", recordHandle, 0);
 
+    // Test with time 2024-07-11T09:15:10.010
+    genericFlow->msecFirst = ParseTime8601("2024-07-11T09:15:10.010");
+    CheckFilter("first seen 2024-07-11T09:15:10.010", recordHandle, 1);
+    CheckFilter("first seen > 2024-07-11T09:15:10.010", recordHandle, 0);
+    CheckFilter("first seen > 2024-07-11T09:15:10.009", recordHandle, 1);
+    CheckFilter("first seen > 2024-07-10T09:15:10.010", recordHandle, 1);
+    CheckFilter("first seen < 2024-07-11T09:15:10.011", recordHandle, 1);
+    CheckFilter("first seen < 2024-07-11T09:15:10.010", recordHandle, 0);
+    CheckFilter("first seen < 2024-07-10T09:15:10.010", recordHandle, 0);
+
+    genericFlow->msecLast = ParseTime8601("2024-07-11T09:15:10.010");
+    genericFlow->msecFirst = 0;
+    CheckFilter("last seen 2024-07-11T09:15:10.010", recordHandle, 1);
+    CheckFilter("last seen > 2024-07-11T09:15:10.010", recordHandle, 0);
+    CheckFilter("last seen > 2024-07-11T09:15:10.009", recordHandle, 1);
+    CheckFilter("last seen > 2024-07-10T09:15:10.010", recordHandle, 1);
+    CheckFilter("last seen < 2024-07-11T09:15:10.011", recordHandle, 1);
+    CheckFilter("last seen < 2024-07-11T09:15:10.010", recordHandle, 0);
+    CheckFilter("last seen < 2024-07-10T09:15:10.010", recordHandle, 0);
+
     genericFlow->proto = IPPROTO_TCP;
     genericFlow->tcpFlags = 1;  // FIN
     CheckFilter("flags F", recordHandle, 1);
@@ -388,6 +408,22 @@ static void runTest(void) {
     CheckFilter("flowdir egress", recordHandle, 1);
     CheckFilter("flowdir ingress", recordHandle, 0);
 
+    // EXcntFlowID
+    PushExtension(recordHeaderV3, EXcntFlow, cntFlow);
+    MapRecordHandle(recordHandle, recordHeaderV3, 1);
+    cntFlow->flows = 10;
+    CheckFilter("flows 10", recordHandle, 1);
+    CheckFilter("flows > 10", recordHandle, 0);
+    CheckFilter("flows < 10", recordHandle, 0);
+    cntFlow->outPackets = 1234;
+    CheckFilter("out packets 1234", recordHandle, 1);
+    CheckFilter("out packets > 1234", recordHandle, 0);
+    CheckFilter("out packets < 1234", recordHandle, 0);
+    cntFlow->outBytes = 5678;
+    CheckFilter("out bytes 5678", recordHandle, 1);
+    CheckFilter("out bytes > 5678", recordHandle, 0);
+    CheckFilter("out bytes < 5678", recordHandle, 0);
+
     // EXvLan
     PushExtension(recordHeaderV3, EXvLan, vlan);
     MapRecordHandle(recordHandle, recordHeaderV3, 1);
@@ -879,7 +915,7 @@ static void runTest(void) {
     // ja4
     ja4_t *ja4 = malloc(sizeof(ja4_t) + SIZEja4String + 1);
     if (ja4 == NULL) {
-        LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno));
+        LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno));
         exit(255);
     }
     ja4->type = TYPE_JA4;
@@ -1009,12 +1045,34 @@ static void runTest(void) {
     // EXipInfo
     PushExtension(recordHeaderV3, EXipInfo, ipInfo);
     MapRecordHandle(recordHandle, recordHeaderV3, 1);
-    ipInfo->ttl = 100;
+    ipInfo->minTTL = 36;
+    ipInfo->maxTTL = 48;
     ipInfo->fragmentFlags = flagDF;
-    CheckFilter("ttl 100", recordHandle, 1);
-    CheckFilter("ttl > 99", recordHandle, 1);
-    CheckFilter("ttl 200", recordHandle, 0);
+    CheckFilter("min ttl 36", recordHandle, 1);
+    CheckFilter("min ttl > 30", recordHandle, 1);
+    CheckFilter("min ttl < 37", recordHandle, 1);
+    CheckFilter("min ttl 64", recordHandle, 0);
+    CheckFilter("min ttl < 30", recordHandle, 0);
+    //
+    CheckFilter("max ttl 48", recordHandle, 1);
+    CheckFilter("max ttl > 40", recordHandle, 1);
+    CheckFilter("max ttl < 50", recordHandle, 1);
+    CheckFilter("max ttl 64", recordHandle, 0);
+    CheckFilter("max ttl < 30", recordHandle, 0);
+    CheckFilter("max ttl 48", recordHandle, 1);
+    //
+    CheckFilter("ttl 36", recordHandle, 1);
+    CheckFilter("ttl 48", recordHandle, 1);
+    CheckFilter("ttl > 40", recordHandle, 1);
+    CheckFilter("ttl < 50", recordHandle, 1);
+    CheckFilter("ttl 64", recordHandle, 0);
+    CheckFilter("ttl < 30", recordHandle, 0);
 
+    CheckFilter("ttl equal", recordHandle, 0);
+    CheckFilter("not ttl equal", recordHandle, 1);
+    ipInfo->maxTTL = 36;
+    CheckFilter("ttl equal", recordHandle, 1);
+    CheckFilter("not ttl equal", recordHandle, 0);
     printf("DONE.\n");
 }  // End of runTest
 
